#include "regexp.h"

/*
 * Replace the matching substring with the subsitution
 * string and write the modified source string to dst.
 * This function can be called with dst == 0 to calculate the
 * length of resulting string.
 */
int regnreplace(r, subst, slen, dst0)
	const regexp *r;
	const char *subst;
	int slen;
	char *dst0;
{
	register int len;
	register const char *src;
	register char *dst = dst0;

	if( r == 0 || r->magic != REGEXP_MAGIC || r->matchstr == 0 ) {
		regerror("no regular expression");
		return;
	}

	/* Copy the prefix */
	src = r->matchstr;
	len = r->startp[0] - src;
	if( len < 0 ) {
	fouled: regerror("match string damaged");
		return;
	} else if( len > 0 ) {
		if( dst0 != 0 )
			memcpy(dst, src, len);
		src += len;
		dst += len;
	}

	/* Skip the matched substring */
	len = r->endp[0] - r->startp[0];
	if( len < 0 )
		goto fouled;
	src += len;

	/* Do actual substitution */
	len = regnsub(r, subst, slen, dst0 ? dst : 0);
	if( len == -1 )
		return;
	dst += len;

	/* Finally, copy the suffix (without the trailing \0) */
	len = (r->matchstrlen + r->matchstr) - src;
	if( len < 0 )
		goto fouled;
	if( dst0 != 0 && len != 0 )
		memcpy(dst, src, len);
	dst += len;
	return dst - dst0;
}
