#!/usr/bin/perl

$VERBOSE = 0;
$Comment  = '!';        # default comment char
$PrepChar = '%';        # default preprocessor directive char


@PrepFunc = ("define", "directory", "undef", "include");
%DefList = ();
$MAXLEVEL = 20;
$LEVEL = 0;

$0 =~ m/([^\/]+)$/;
$progname = $1;
$usage = "Usage: $progname [-c comment_char] [-d define_char] [-v] [files]\n";
&getopts ('c:d:v') || die $usage;
$Comment  = $opt_c if ($opt_c);
$PrepChar = $opt_d if ($opt_d);
$VERBOSE  = STDERR if ($opt_v);

if ($#ARGV < 0) {
    &process ("-", 0);
}
else {
    foreach (@ARGV) {
	&process ($_, 0);
    }
}

sub process {
    local ($FILE) = @_[0];
    local ($I);
    $I = "FILE_".$LEVEL;    #  filehandle -   :-(

    if ($FILE eq "-") { $I = STDIN; $FILE = "STDIN"; }
    else { open ($I, $FILE) || die ("Can't open file \"$FILE\": $!"); }

    while (<$I>) {
	$s = $_;
      #    
	if ($s =~ /^$Comment/) {
	    print STDOUT $s;
	    next;
	}
      #     
	$s =~ s/($Comment)(.*)([\r\n])$/$3/;

      #  
	if ($s =~ /^$PrepChar/) {
	    local ($done) = 0;
	    $s =~ s/[ \t\r\n]+$//;
	    for $func (@PrepFunc) {
		if ($s =~ s/$PrepChar$func[ \t]//) {
		    print STDOUT "$Comment$Comment $func $s\n";
		    local ($arg1, $arg2) = split (/[ \t]+/, $s, 2);
		    die ("$FILE: Directive $func can't arguments")
			if ($arg1 eq '');
		    &$func ($FILE, $arg1, $arg2);
		    $done++;
		    last;
		}
	    }
	    if ($done == 0) {
		print $VERBOSE "$FILE: unknown MACRO: $s\n";
	    }
	    next;
	}

      #  defines
	foreach $d (keys %DefList) {
	    $v = $DefList {$d};
	    $s =~ s/(^|[ \t]+)($d)($|[ \t\n\r]+)/$1$v$3/;
	}
	print STDOUT $s;
    }

    close ($I);
}

sub define {
    local ($FILE, $key, $val) = @_;
    $val =~ s/^"//;     #    
    $val =~ s/"$//;     #         (  :-).
    $DefList {$key} = $val;
    print $VERBOSE "$FILE: Define \"$key\" -> \"$val\"\n";
}

sub undef {
    local ($FILE, $key) = @_;
    delete ($DefList {$key});
    print $VERBOSE "$FILE: UnDef \$key\"\n";
}

sub directory {
    local ($FILE, $dir) = @_;
    chdir ($dir) || die ("FILE: Can't chdir to $dir");
}

sub include {
    local ($FILE, $file) = @_;
    $file =~ s/^["<]//;     #    
    $file =~ s/[>"]$//;     #         (  :-).
    die ("$FILE: Level too large for next include $file")
	    if ($LEVEL++ > $MAXLEVEL);
    print STDOUT "$Comment$Comment$Comment START include file \"$file\"\n";
    &process ($file);
    print STDOUT "$Comment$Comment$Comment STOP include file \"$file\"\n";
}

# =========================================================
sub getopts {
    local($argumentative) = @_;
    local(@args,$_,$first,$rest);
    local($errs) = 0;

    @args = split( / */, $argumentative );
    while(@ARGV && ($_ = $ARGV[0]) =~ /^-(.)(.*)/) {
	($first,$rest) = ($1,$2);
	$pos = index($argumentative,$first);
	if($pos >= 0) {
	    if($args[$pos+1] eq ':') {
		shift(@ARGV);
		if($rest eq '') {
		    ++$errs unless @ARGV;
		    $rest = shift(@ARGV);
		}
		eval "\$opt_$first = \$rest;";
		push( @EXPORT, "\$opt_$first" );
	    }
	    else {
		eval "\$opt_$first = 1";
		push( @EXPORT, "\$opt_$first" );
		if($rest eq '') {
		    shift(@ARGV);
		}
		else {
		    $ARGV[0] = "-$rest";
		}
	    }
	}
	else {
	    print STDERR "Unknown option: $first\n";
	    ++$errs;
	    if($rest ne '') {
		$ARGV[0] = "-$rest";
	    }
	    else {
		shift(@ARGV);
	    }
	}
    }
    $errs == 0;
}
