/*
 * (C) RELCOM, 1997
 * This part of code was developed in Relcom by Julia Pasynkova and Aleksei Roudnev,
 * /
/*
 * Writing status file for ALL lines
 * Format:  /should be changed in some short time/
 * # - omment
 * . - END
 * <Type> <Fields>
 * - fiend delimiter - TABS
 * - any field can be followed by: /<CLASS><TIME>
 *   when <CLASS> - E - Error, W - Warning, 0 - critical time
 *        <TIME> - time since this attribute got into this state
 *
 *
 * T TIME NUMBER
 *       (UNIX time and current number of the file version)
 * R name status C<cpu> M<mem> T<time> P<mem> I<mem> V<mem> F<mem> U<mem>
 *               P<free,used,mseg> I<free,used,mseg> V<free,used,mseg>
 *               F<free,used,mseg> U<free,used,mseg>
 * - router, status=UP, DOWN
 * - cpu, percents
 * - mem - free memory, K
 * - time - time of live
 * - P - PROCESSOR memory, K
 * - I - I/O memory, K
 * - V - PCI memory, K
 * - F - FAST memory, K
 * - U - MULTIBUS memory, K
 *
 * I <name> <status> <dir> <comment> <BW> [<IN%> <INpak> <OUT%> <OUTpak> <INerr%> <INdrop%> <OUTerr%> <OUTdrops%>]
 * - interfaces
 *   status - U - up, D - down, L - looped, N - unexistent, T - no data.
 *   Speed - bps
 *   dir - directory name
 *   comment - interface name
 * Another parameters are in %f4.1 form (double)
 * IN%, OUT% - link load, rercents
 * INpak, OUTpak - packets/second
 * INerr%, OUTerr% - errors, percents
 * INdrops%, OUTdrops% - in and out drops, percents
 * A - Async line, for future use
 * BN - there is no configured bgp peer
 * BY - there is configured bgp peer
 */


#include <sys/file.h>
#include "manifest.h"

# ifndef __FreeBSD__
extern char *sys_errlist[];
# endif
extern int errno;

/*    BGP */
static struct bgp_ifsum {
       char *b_status;
       char *b_reason;
} bgp_states[] = {
{/* 0 BGP_NEVER         */ "N",   "never/U"},
{/* 1 BGP_IDLE          */ "D",   "idle/E"},
{/* 2 BGP_CONNECT       */ "D",   "connect/E"},
{/* 3 BGP_ACTIVE        */ "D",   "active/E"},
{/* 4 BGP_OPENSENT      */ "U",   "opensent/W"},
{/* 5 BGP_OPENCONF      */ "U",   "openconf/W"},
{/* 6 BGP_ESTABLISHED   */ "U",   "established"},
{/* 7 BGP_INAC          */ "D",   "inact/E"},
			  { 0,    0 }
};

/* Do ifsum dump */
void
write_ifsums()
{
	struct GATEWAY *g;
	struct IF *i;
	struct BGP *b;
	struct MEMPOLL *m;
	u_long curr_tt;
	long ch_tt;
	static unsigned SERIAL;
	static struct SNMP_ACTION a;
	void net_state();
	static bad_stats;

	if ( !fifstat ) return;

	/* Try to setup exclusive lock */

	if(flock(fileno(fifstat),LOCK_NB|LOCK_EX) == -1) {
		if ( bad_stats++ < 4 ) {
		       a.a_f = net_state;
		       a.a_time = time(0) + 1; /*   */
		       timeout(&a);
		       return;
		};
		fprintf(stderr,"%s: Can't setup exclusive lock to ifsum file %s\n", av0, sys_errlist[errno]);
	}
	bad_stats = 0;

	fseek(fifstat,0L,0);
	/*
 * 1.  SERIAL   
 */
	curr_tt = time(0);
	fprintf(fifstat,"T\t%u\t%u\n",curr_tt,++SERIAL);
	/*
 *   
 */
	for( g = gwlist ; g ; g = g->g_next ) {
		if ( g->g_sname )
			fprintf(fifstat,"R\t%s(%s)\t",g->g_sname,g->g_name);
		else
			fprintf(fifstat,"R\t%s\t",g->g_name);
		if( g->g_alive != G_UP )
		{
			fprintf(fifstat,"D/E%d\n",curr_tt - g->g_lastmsg);
		} else {
			if( curr_tt - g->g_lastchst < TIME_INSTABLE )
				fprintf(fifstat,"U/O%d",curr_tt - g->g_lastmsg);
			else 
				fprintf(fifstat,"U");

			fprintf(fifstat,"\tC%d",g->g_cpubusy);
			if ( g->g_cpubusy > gw_maxcpu )
				fprintf(fifstat,"/W%d", curr_tt - g->g_cpubusytime);
			fprintf(fifstat,"\tM%d",g->g_freemem);
			if ( g->g_freemem < gw_minmem && g->g_freemem > 0 )
				fprintf(fifstat,"/W%d", curr_tt - g->g_memlowtime);
			fprintf(fifstat,"\tT%d", g->g_uptime/100 );


			for( m = g->g_mem; m; m = m->mem_next ) {
			  if (m->mem_valid != MEM_VALID_OK)
				continue;
			  fprintf(fifstat,"\tm%s,%d",m->mem_name,m->mem_free);
			  if( m->mem_free < m->mem_minmem )
				fprintf(fifstat,"/W%d",curr_tt - m->mem_memlowtime);
                          fprintf(fifstat,",%d,%d",m->mem_used,m->mem_maxseg);
			}
			fprintf(fifstat,"\n");
		};

		for( i = g->g_ifs ; i < &g->g_ifs[g->g_nifs] ; i++ )
		{
			fprintf(fifstat,"I\t%s\t",i->i_name);
			/*
		  *    ,  N 
CK_SH
		  */
			if( i->i_index < 0 )
			{
				fprintf(fifstat,"N\t%s\t%s\n",i->i_logdir, (i->i_comment?i->i_comment : i->i_logdir) );
				continue;
			};
			/* no data */
			if( i->i_cs.l_timestamp == 0) {
				fprintf(fifstat,"T\t%s\t%s\n",i->i_logdir, (i->i_comment?i->i_comment : i->i_logdir) );
				continue;
			};
			/*
		  *       
		  */
			ch_tt = i->i_cs.l_reftime - (g->g_uptime - i->i_cs.l_LastChange) / 100;
			/*
		  *   .    DOWN,  ERROR
		  *    SPEED (    )
		  */
			if ( i->i_cs.l_OperStatus == IF_DOWN ) {
				if (ch_tt > i->i_cs.l_lastchtt) {
					ch_tt=i->i_cs.l_lastchtt;
					dprintf(("GW: g->g_sname IF: i->i_name change tt %d to %d", ch_tt, i->i_cs.l_lastchtt
					    ));
				}
				fprintf(fifstat,"D/E%d\t%s\t%s\t%d\n",curr_tt - ch_tt, i->i_logdir,(i->i_comment?i->i_comment
				    : i->i_logdir) , i->i_cs.l_Speed );
				continue;
			};
			if ( i->i_cs.l_OperStatus == IF_SHUTDOWN ) {
				if (ch_tt > i->i_cs.l_lastchtt) {
					ch_tt=i->i_cs.l_lastchtt;
					dprintf(("GW: g->g_sname IF: i->i_name change tt %d to %d", ch_tt, i->i_cs.l_lastchtt
					    ));
				}
				fprintf(fifstat,"S/E%d\t%s\t%s\t%d\n",curr_tt - ch_tt, i->i_logdir,(i->i_comment?i->i_comment
				    : i->i_logdir) , i->i_cs.l_Speed );
				continue;
			};
			if ( i->i_cs.l_OperStatus == IF_INACTIVE ) {
				if (ch_tt > i->i_cs.l_lastchtt) {
				       /*  ch_tt = i->i_cs.l_lastchtt; */
					dprintf(("GW: g->g_sname IF: i->i_name change tt %d to %d", ch_tt, i->i_cs.l_lastchtt
					    ));
				}
				fprintf(fifstat,"I/E%d\t%s\t%s\t%d\n",curr_tt - ch_tt, i->i_logdir,(i->i_comment?i->i_comment
				    : i->i_logdir) , i->i_cs.l_Speed );
				continue;
			};
			if ( i->i_cs.l_OperStatus == IF_UP ) {
				if( curr_tt - i->i_cs.l_lastchtt < TIME_INSTABLE )
					fprintf(fifstat,"U/O%d",curr_tt - i->i_cs.l_lastchtt);
				else
					fprintf(fifstat,"U");

				fprintf(fifstat,"\t%s\t%s\t" , i->i_logdir,(i->i_comment?i->i_comment : i->i_logdir));
			}
			else if ( i->i_cs.l_OperStatus == IF_INAC) {
				fprintf(fifstat,"N\t%s\t%s\t\n" , i->i_logdir,(i->i_comment?i->i_comment : i->i_logdir));
				continue;
			}
			else {
				if ( i->i_cs.l_OperStatus == IF_TESTING ) {
					fprintf(fifstat,"L/W%d\t%s\t%s\t",curr_tt - ch_tt, i->i_logdir, (i->i_comment?i->i_comment
					    : i->i_logdir));
				} else {
					fprintf(fifstat,"?/W%d\t%s\t%s\t",curr_tt - ch_tt, i->i_logdir, (i->i_comment?i->i_comment
					    : i->i_logdir));
				};
			};
			/*
		   *     .
		   *      WARNING
		   */
# define FPT(d,tr,t,c) (((t) && (tr) > 0.0 && (d) > (tr)) ? fprintf(fifstat,"\t%.1f/%c%d",d,c,curr_tt - t) : fprintf(fifstat,"\t%.1f",d) )
			fprintf(fifstat,"%d",i->i_cs.l_Speed); /* \t FPT   */

			FPT(i->i_out.s_inutil, i->i_maxinutil,   i->i_overloadtime,'W');
			FPT(i->i_out.s_inpckts, 0.0,              0,               'W');
			FPT(i->i_out.s_oututil, i->i_maxoututil, i->i_overloadtime,'W');
			FPT(i->i_out.s_outpckts,  0.0,            0,               'E');
			/* :    ,    -  W  E */
			FPT(i->i_out.s_inerrors, err_threshold,  i->i_errorstime,( (i->i_out.s_inutil > 10 || i->i_out.s_inerrors > 40 )  ?'E':'W') );
			/* :     .    ,     */
			FPT(i->i_out.s_indrops , (i->i_map.m_key == READ_IFDESCR ?  drop_threshold : 85 ), i->i_dropstime, 'W');
			FPT(i->i_out.s_outerrors,err_threshold,  i->i_errorstime,  'E');
			FPT(i->i_out.s_outdrops ,drop_threshold, i->i_dropstime,   'W');
			fprintf(fifstat,"\n");
		}
		/* BGP - B U established name ADDRESS AS TIME */
		for(b=g->g_bgp;b;b=b->bgp_next) {
			if( b->bgp_valid == BGP_VALID_BAD)
				fprintf(fifstat,"B\tN\t%s\t%s\n",b->bgp_name,b->bgp_ip);

			else {
			       int i = b->bgp_state;
			       if ( i >= 0 &&  i < sizeof(bgp_states)/sizeof(struct bgp_ifsum) )
			       {
				if ( index(bgp_states[i].b_reason,'/') )
				    fprintf(fifstat,"B\t%s\t%s%d\t%s\t%s\t%d\t%lu\n",
					bgp_states[i].b_status,bgp_states[i].b_reason,b->bgp_time,b->bgp_name,b->bgp_ip,b->bgp_as,b->bgp_time);
				else
				    fprintf(fifstat,"B\t%s\t%s\t%s\t%s\t%d\t%lu\n",
					bgp_states[i].b_status,bgp_states[i].b_reason,b->bgp_name,b->bgp_ip,b->bgp_as,b->bgp_time);
				} else {
				    fprintf(fifstat,"B\tU\t%d\t%s\t%s\t%d\t%lu\n",
					   i,b->bgp_name,b->bgp_ip,b->bgp_as,b->bgp_time);
			       };
		       };
		}
	}
	fprintf(fifstat,".\n");
	fflush(fifstat);
	ftruncate(fileno(fifstat),ftell(fifstat));

	if(flock(fileno(fifstat),LOCK_UN) == -1)
		fprintf(stderr,"%s: Can't remove exclusive to ifsum file %s\n", 			av0, sys_errlist[errno]);


}

# ifdef LOCK_FCNTL
#include <errno.h>
int
flock(fd, operation)
int fd, operation;
{
	int op, ret;
	struct flock arg;
	extern int errno;
# ifdef __STDC__
	extern int fcntl(int, int, ...);
# else /* !__STDC__ */
	extern int fcntl();
# endif /* __STDC__ */
	op = (LOCK_NB & operation) ? F_SETLK : F_SETLKW;
	arg.l_type = (LOCK_EX & operation) ? F_WRLCK :
		(LOCK_SH & operation) ? F_RDLCK : F_UNLCK;
	arg.l_whence = 0;
	arg.l_start = 0;
	arg.l_len = 0;
	arg.l_pid = 0;
	if ((ret = fcntl(fd, op, &arg)) == -1) {
		if (errno == EACCES || errno == EAGAIN)
			errno = EWOULDBLOCK;
	}
	return (ret);
}
# endif
