/*
 * Copyright (C) 1993, US Sprint
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of US Sprint not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * US SPRINT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * US SPRINT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Author: Vadim Antonov
 *
 * $Log: lock.c,v $
 * Revision 1.1.1.1  1999/07/19 18:38:31  alex
 * First revision in cvs
 *
 * Revision 1.7  1999/05/11 15:24:01  alex
 * *** empty log message ***
 *
 * Revision 1.6  1998/03/20  19:02:40  alex
 * RELCOM
 *
 * Revision 1.5  1996/01/10  15:32:19  tim
 * *** empty log message ***
 *
 * Revision 1.4  1995/09/07  09:29:53  tim
 * ???
 *
 * Revision 1.4  1995/09/07  09:29:53  tim
 * ???
 *
 * Revision 1.3  1993/09/22  23:25:53  avg
 * Changed a diagnostic.
 *
 * Revision 1.3  1993/09/22  23:25:53  avg
 * Changed a diagnostic.
 *
 * Revision 1.2  1993/08/11  19:40:10  avg
 * Added copyright.
 *
 * Revision 1.1  1993/08/10  17:47:57  avg
 * Initial revision
 *
 * Revision 1.1  1993/08/10  17:47:57  avg
 * Initial revision
 *
 */
#ifndef lint
static char rcsid[] = "$Id: lock.c,v 1.1.1.1 1999/07/19 18:38:31 alex Exp $";
#endif

#include "manifest.h"
#include <sys/stat.h>
#include <errno.h>

# ifndef __FreeBSD__
extern char *sys_errlist[];
extern int errno;
# endif

char lockfn[512];

static u_long startt;

void touch_lock();

/*
 * Check the presense of the fresh lock file
 * Returns 1 if there is another daemon alive.
 */
int check_lock( char *lf )
{
	u_long tt, t0;
	struct stat st;
      
        if (lf == 0 ) {
          if ( lock_file[0] ) {
	    strcpy(lockfn, lock_file);
          }
          else {
	    strcpy(lockfn, base_dir);
	    strcat(lockfn, "/LOCK");
          }
        }
        else {
	  strcpy(lockfn, lf);
        }

	t0 = 0;
	startt = time(0);
	while( stat(lockfn, &st) >= 0 ) {
		tt = time(0);
		if( (u_long) st.st_mtime < tt - LOCK_EXPIRATION )
			break;
		if( t0 == 0 )
			t0 = st.st_mtime;
		else if( t0 != st.st_mtime )    /* somebody touched it! */
			return 1;
		sleep(3);
	}
	touch_lock((struct SNMP_ACTION *)0);
	return 0;
}

/*
 * Touch the lock file
 */
void
touch_lock(a)
	struct SNMP_ACTION *a;
{
	static struct SNMP_ACTION lock;
	int f;
	char buf[30];

	sprintf(buf, "%d %d\n", getpid(), time(0) - startt);
	f = creat(lockfn, 0644);
	if( f < 0 ) {
		fprintf(stderr, "%s: cannot touch lock file %s %s!\n", 
			av0, lockfn,sys_errlist[errno]);
		exit(1);
	}
	write(f, buf, strlen(buf));
	close(f);
	if( a == 0 )
		lock.a_time = time(0);
	lock.a_time += LOCK_EXPIRATION/2;
	lock.a_f = touch_lock;
	timeout(&lock);
}
