/*
 * Copyright (C) 1993, US Sprint
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of US Sprint not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * US SPRINT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * US SPRINT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Author: Vadim Antonov
 *
 * $Log: rdconf.c,v $
 * Revision 1.1.1.1  1999/07/19 18:38:32  alex
 * First revision in cvs
 *
 * Revision 1.8  1998/03/20  19:02:40  alex
 * RELCOM
 *
 * Revision 1.7  1997/08/26  08:32:49  tim
 * ifsum file
 *
 * Revision 1.6  1996/01/10  15:32:19  tim
 * *** empty log message ***
 *
 * Revision 1.5  1995/09/07  09:29:53  tim
 * output of overhead and errors
 *
 * Revision 1.5  1995/09/07  09:29:53  tim
 * output of overhead and errors
 *
 * Revision 1.4  1995/08/04  14:25:02  tim
 * Added some output (avg line load etc.)
 *
 * Revision 1.3  1993/09/22  23:25:53  avg
 * Added generation of short router names.
 *
 * Revision 1.2  1993/08/11  19:40:10  avg
 * Added copyright.
 * Avoid reading floating numbers from stat records which
 * are out of time interval.
 *
 * Revision 1.1  1993/08/10  17:47:57  avg
 * Initial revision
 *
 * Revision 1.1  1993/08/10  17:47:57  avg
 * Initial revision
 *
 */
#ifndef lint
static char rcsid[] = "$Id: rdconf.c,v 1.1.1.1 1999/07/19 18:38:32 alex Exp $";
#endif

#include "manifest.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

struct GATEWAY *gwlist; /* List of gateways */
int ngateways;          /* The number of gateways polled */
int poll_period;        /* Default polling period */
int aggr_period;        /* Default aggregation period */
char base_dir[512] = ".";                /* Base directory name */
char notify_file[512];  /* File to store notifications in */
char state_file[512];   /* File to store the network state in */
char debug_file[512];   /* File to debug snmpstatd */
char ifsum_file[512];   /* File to store the all interfaces state */
char lock_file[512];    /* File to lock and store proc number */
enum FILE_CHANGE file_change = DFLT_FCH; /* File change period */
float err_threshold = 101.0;    /* Max % of errors on the link */
float drop_threshold = 101.0;   /* Max % of dropped packs on the link */
int fulldomain = 0;     /* Preserve full domain name in Notify/State */

int gw_maxtemp = 200;   /* Max router temperature */
int gw_maxcpu = 101;    /* Max router CPU utilization */
int gw_minmem = -1;     /* Minimal free memory, Kb */
float inutil_threshold  = 0.0;  /* Default max input utilization */
float oututil_threshold = 0.0;  /* Default max output utilization */

enum FILE_CHANGE rdfc();

#define SKIP_PRINTABLE(v) \
		while( *v && *v != ' ' && *v != '\t' ) \
			v++;

#define SKIP_SPACES(v) \
		while( *v == ' ' || *v == '\t' ) \
			*v++ = 0;

#define STREQ(v,t) (!strncmp((v),(t),sizeof(t)-1))

/* Structs for function map */

struct MAP MAP_TABLE[] = {
	{ "NUM", READ_IFINDEX },
	{ 0,     0            }
};



/*
 * Read the configuration file
 * Returns 1 on failure.
 */
int
rdconfig(cfn)
char *cfn;
{
	FILE *f;
	char line[256];
	char *p, *q, *r, *s;
	char *gwn;      /* current gw name */
	char *cmn;      /* community name for gw */
	char *rid;      /* router-id for gw */
	int bgpall;     /* Poll all bgp peers */
	int nif;        /* number of interfaces */
	struct IF ifs[MAXIFS];
	struct BGP *b, *bl, *bh;  /* point to bgp peer */
	struct IF      *ii;
	int poll;       /* polling period for gateway */
	int nl;         /* linenumber for diagnostics */
	int i,j;
	struct GATEWAY *g, *tail;
	struct GATEWAY *fgw; /* already existed the same gateway */
	int ap, pp;
	int Ngw;
	extern int arg_pp, arg_ap;
	float ee, ee1;
	double atof();
	int m_key;        /* Key for MAP function */
	char *m_val;	  /* Val for MAP function */
        struct MEMPOLL *m, *pm; /* MEMPOLL temporary mem */
	int gw_minmem_p = 0;   /* Min router PROCESSOR memory, Kb */
	int gw_minmem_i = 0;   /* Min router I/O memory, Kb */
	int gw_minmem_v = 0;   /* Min router PCI memory, Kb */
	int gw_minmem_f = 0;   /* Min router FAST memory, Kb */
	int gw_minmem_u = 0;   /* Min router MULTIBUS memory, Kb */
	char gw_icmode  = IC_NONE; /* Global CHECK mode */
	/* the same parametrs, but only for one router or link */
	int l_minmem_p = 0;
	int l_minmem_i = 0;
	int l_minmem_v = 0;
	int l_minmem_f = 0;
	int l_minmem_u = 0;
	int l_icmode   = IC_NONE;
	char icmode = IC_NONE;


	/* Open the config file */
	if( cfn == NULL )
		cfn = CONFIG_FILE;
	if( (f = fopen(cfn, "r")) == NULL ) {
		fprintf(stderr, "%s: cannot read config file %s\n", av0, cfn);
		return 1;
	}

	strcpy(debug_file,"/tmp/snmpstatd.debug");

	gwlist = tail = 0;
	pp = arg_pp;
	ap = arg_ap;

	/*
	 * Parse the file line by line
	 */
	nl = 0;
	gwn = 0;
	Ngw = 0;
	while( fgets(line, sizeof line, f) ) {
		nl++;

		/* Skip comments */
		if( p = index(line, '\n') )
			*p = 0;
		if( p = index(line, '#') )
			*p = 0;
		p = line;
		SKIP_SPACES(p);
		if( *p == 0 )
			continue;

		/* Trim trailing spaces */
		q = p + strlen(p);
		while( q > p && (*--q == ' ' || *q == '\t') )
			*q = 0;

		m_val=NULL;
		m_key=-1;

		/* Is this a link declaration? */
		if( gwn ) {
			/* End of link declarations? */
			if( p[0] == '}' ) {
				if( p[1] ) {
					s = "garbage after }";
					goto synterr;
				}

				/* Allocate memory for the gw structure */
				if (!fgw) {
					i = sizeof(struct GATEWAY) +
					    sizeof(struct IF) * (nif-1);

					g = (struct GATEWAY *) malloc(i);
					if( g == 0 )
						goto nomem;

					bzero((char *)g, i);
					g->g_next  = 0;
					g->g_name  = gwn;
					g->g_sname = gwn;
					g->g_bgpall = bgpall;
					g->g_bgp   = bh;      /* head of BGP list */
                                        g->g_mem   = 0;
                                        if (gw_minmem_p > 0 || l_minmem_p >0) {
					  i = sizeof(struct MEMPOLL);
                                          m = (struct MEMPOLL *) malloc(i);
				          if(!m)
						goto nomem;
					  if (!g->g_mem)
                                             g->g_mem = m;
					  else
					     pm->mem_next=m;
/* temporary hack. delete it */
                                          m->mem_name=mempolls[1];

					  m->mem_minmem = (l_minmem_p > 0) ? l_minmem_p : gw_minmem_p;
					  m->mem_type   = 1;
					  m->mem_next   = NULL;
                                          pm = m; 
                                          l_minmem_p=0; /* clear old */
                                        }
                                        if (gw_minmem_i > 0 || l_minmem_i > 0) {
                                           i = sizeof(struct MEMPOLL);
                                           m = (struct MEMPOLL *) malloc(i);
					   if(!m)
                                                goto nomem;
                                          if (!g->g_mem) 
                                             g->g_mem = m;
                                          else
                                             pm->mem_next=m;

                                          m->mem_name=mempolls[2];

                                          m->mem_minmem = (l_minmem_i >0) ? l_minmem_i : gw_minmem_i;
                                          m->mem_type   = 2; /* i/o memory */
                                          m->mem_next   = NULL;
                                          pm = m;
					  l_minmem_i=0;
                                        }
                                        if (gw_minmem_v > 0 || l_minmem_v > 0) {
                                           i = sizeof(struct MEMPOLL);
                                           m = (struct MEMPOLL *) malloc(i);
					   if(!m)
                                                goto nomem;
                                          if (!g->g_mem) 
                                             g->g_mem = m;
                                          else
                                             pm->mem_next=m;

				          m->mem_name=mempolls[3];

                                          m->mem_minmem = (l_minmem_v >0) ? l_minmem_v : gw_minmem_v ;
                                          m->mem_type   = 3;
                                          m->mem_next   = NULL;
                                          pm = m; 
					  l_minmem_v=0;
                                        }
                                        if (gw_minmem_f > 0 || l_minmem_f >0 ) {
                                           i = sizeof(struct MEMPOLL);
                                           m = (struct MEMPOLL *) malloc(i);
					   if(!m)
                                                goto nomem;
                                          if (!g->g_mem) 
                                             g->g_mem = m;
                                          else
                                             pm->mem_next=m;

				          m->mem_name=mempolls[4];
	
                                          m->mem_minmem=(l_minmem_f >0) ? l_minmem_f : gw_minmem_f; 
                                          m->mem_type   = 4;
                                          m->mem_next   = NULL;
                                          pm = m; 
					  l_minmem_f=0;
                                        }

                                        if (gw_minmem_u > 0 || l_minmem_u > 0 ) {
                                           i = sizeof(struct MEMPOLL);
                                           m = (struct MEMPOLL *) malloc(i);
					   if(!m)
                                                goto nomem;
                                          if (!g->g_mem) 
                                             g->g_mem = m;
                                          else
                                             pm->mem_next=m;

					  m->mem_name=mempolls[5];

                                          m->mem_minmem = (l_minmem_u > 0) ? l_minmem_u : gw_minmem_u;
                                          m->mem_type   = 5; 
                                          m->mem_next   = NULL;
                                          pm = m; 
					  l_minmem_u=0;
                                        }

					if(rid)
						g->g_sname = rid;

					else {
						/* Is it a numeric address? */
						for( q = gwn ; *q &&
						    ((*q >= '0' && *q <= '9') || *q == '.') ; q++ ) ;
						if (*q) {   /* try to get IP address from DNS */
							struct hostent *hp;

							hp = gethostbyname(gwn);
							if (hp == NULL) {
								s = "unknown host";
								goto synterr;
							}
							if ( ! (g->g_name = (char *) malloc (3*4+3+1)) )
								goto nomem;

/*adbr
                					strcpy (g->g_name, inet_ntoa(hp->h_addr));
*/
                                                        strcpy (g->g_name, hp->h_name); /*adbr */

							if( !fulldomain && (q = index(gwn, '.'))) {
								*q = 0;

								/* Should check for uniqueness of short names here */
								p = (char*) malloc(strlen(gwn)+1);
								if( p == 0 )
/* code folded from here */
	goto nomem;
/* unfolding */
								strcpy(p, gwn);
								*q = '.';
								g->g_sname = p;
							}
						} else {    /* try to get a name from DNS */
							struct hostent *hp;
							struct in_addr gia;
							gia.s_addr = inet_addr (gwn);
							if (gia.s_addr == -1) {
								s = "invalid gateway address";
								goto synterr;
							}
							hp = gethostbyaddr ((char *)&gia, sizeof(gia), AF_INET);
							if (hp) {
								g->g_sname = (char *)malloc(strlen(hp->h_name)+1);
								if (!g->g_sname)
/* code folded from here */
	goto nomem;
/* unfolding */
								strcpy(g->g_sname, hp->h_name);
								/* Should check for uniqueness of short names here */
								if(!fulldomain && (q = index(g->g_sname, '.')) )
/* code folded from here */
	*q = 0;
/* unfolding */
							}
						}
					}

					g->g_cname = cmn;
					g->g_poll = poll;
					g->g_nifs = nif;
					g->g_n = Ngw;
					if( nif > 0 )
						memcpy(g->g_ifs, ifs, sizeof(struct IF)*nif);

					/* Queue the gateway to the list */
					if( tail == 0 )
						gwlist = g;
					else
						tail->g_next = g;
					tail = g;
				} /*if(!fgw)*/
				else if(nif==0)
				{ 
					gwn=NULL; 
					continue; 
				}
				else {
					struct GATEWAY *p;
					i=sizeof(struct GATEWAY) + sizeof(struct IF)*(nif-1+fgw->g_nifs);
					if((g=(struct GATEWAY *)realloc((char *)fgw,i)) == NULL)
						goto nomem;
					if(g != fgw) {
						bzero((char*)g,i);
						memcpy(g,fgw,sizeof(struct GATEWAY) + sizeof(struct IF)*(fgw->g_nifs-1));


						for(p=gwlist,i=0;p;p=p->g_next){
							if(i) {
								p->g_n--;
								continue;
							}
							if(fgw->g_n - p->g_n == 1)
							{
								if(fgw->g_next)	p->g_next=fgw->g_next;
								else { 
/* code folded from here */
	p->g_next=g; 
	break; 
/* unfolding */
								}
								i++;
							}
						}
						g->g_next=0;
					}
					g->g_n=Ngw;
					if (nif > 0)
						memcpy(&g->g_ifs[g->g_nifs],ifs,sizeof(struct IF)*nif);
					g->g_nifs=fgw->g_nifs + nif;
					if( fgw==tail ) {
						if(fgw==gwlist) gwlist=g;
						else p->g_next=g;
					}
					else tail->g_next = g;
					tail = g;
					free(fgw);
				}
				gwn=NULL;
				continue;
			}

			/* Extract the link name, log dir name and comment */
			q = p;
			SKIP_PRINTABLE(q);
			if( *q == 0 ) {
				s = "missing log directory name";
				if( !strcmp(p, "POLL") )
					s = "missing argument";
				goto synterr;
			}
			SKIP_SPACES(q);
			r = q;
			SKIP_PRINTABLE(q);
			SKIP_SPACES(q);

			/* Polling interval specification? */
			if( !strcmp(p, "POLL") ) {
				if( *q ) {
					s = "extra argument for POLL";
					goto synterr;
				}
				poll = rdtime(r);
				if( poll < 1 ) {
					s = "bad poll time";
					goto synterr;
				}
				continue;
			}
			/* Utilization threshold specification and ic mode */
			ee = inutil_threshold;
			ee1 = oututil_threshold;
			l_icmode = gw_icmode;
			while( STREQ(q,"MAXINUTIL")
			    || STREQ(q,"MAXOUTUTIL")
			    || STREQ(q,"MAXUTIL") 
			    || STREQ(q,"CHECK") )
			{
				char *l, *v;
				l = q;
				SKIP_PRINTABLE(q);
				SKIP_SPACES(q);
				if (*q == 0) {
					s = "line option mission arg: (MAXINUTIL percent, ..., CHECK in, CHECK atmin) ";
					goto synterr;
				}
				v = q;
				SKIP_PRINTABLE(q);
				SKIP_SPACES(q);
				if( !strcmp (l, "MAXINUTIL") ) {
					ee = atof(v);
					if( ee < 0.0 || ee > 200.0 ) {
						s = "input utilization threshold is not from 0.0 to 200.0";
						goto synterr;
					}
					continue;
				}
				if( !strcmp (l, "CHECK") ) {
					l_icmode = IC_INACTIVE;
					if ( v[0] == 'a') l_icmode = IC_INACTIVE_ATM;
					continue;
				}if( !strcmp (l, "MAXOUTUTIL") ) {
					ee1 = atof(v);
					if( ee1 < 0.0 || ee1 > 200.0 ) {
						s = "output utilization threshold is not from 0.0 to 200.0";
						goto synterr;
					}
					continue;
				}
				if( !strcmp (l, "MAXUTIL") ) {
					ee = atof(v);
					if( ee < 0.0 || ee > 200.0 ) {
						s = "utilization threshold is not from 0.0 to 200.0";
						goto synterr;
					}
					ee1 = ee;
					continue;
				}
			}

			if (strncmp(p,"INT",strlen("INT"))==0) {
				/* start map function INT(NUM,INDEX) for example */
				char *m;
				int ii,l;

				for (ii=0, m=(char *)strtok(p,",()"); m && ii< 3;
				    m=(char *)strtok(NULL, ",)("), ii++) {
					if(m == NULL) {
						s = "function INT error. function must be INT(NUM,INDEX) ";
						goto synterr;
					}
					if(ii == 0 && !STREQ(m,"INT")){
						s = "sorry. there is only INT function";
						goto synterr;
					}
					if(ii == 1) {
						l=0;
						while(MAP_TABLE[l].m_key) {
							if(strcmp(MAP_TABLE[l].m_val,m) == 0 ) {
								m_key=MAP_TABLE[l].m_key;
								break;
							}
							l++;
						}
						if(m_key == -1) {
							s = "function INT error. There is no such key";
							goto synterr;
						}
					}
					if(ii == 2) {
						m_val=(char *) calloc(1, strlen(m)+1);
						if(!m_val ) {
							perror("Malloc m_val");
							exit(1);
						}
						strcpy(m_val,m);
					}
				}
				m=q;
				SKIP_PRINTABLE(q);
				SKIP_SPACES(q);
				if (*q == 0) {
					s = "bad description line after INT function";
					goto synterr;
				}
				p=r;
				r=m;
			}
			if (STREQ(p,"BGP") && !bgpall) {
				int si;

				if (STREQ(r,"ALL")) {
					/* check if bgpall already installed */
					/*if(g && g->g_bgpall) continue;*/
					bgpall=1;
					continue;
				}

				for (b=g->g_bgp;b;b=b->bgp_next) {
				  if(!strncmp(b->bgp_ip,r,strlen(b->bgp_ip))
					&& strlen(b->bgp_ip) == strlen(r) ){
				    fprintf(stderr, "%s: ignore repeated bgp %s line %d \n",
					 gwn, b->bgp_ip, nl);
				    goto bcont;
				  }	
				}

				si = strlen(r) + strlen(q) + 2;
				s  = (char *) malloc(si);
				if( s == 0 )
					goto nomem;
				bzero(s,si);
				si = sizeof(struct BGP);
				b  =(struct BGP *)malloc(si);
				if( b == 0 )
					goto nomem;
				bzero(b,si);
				strcpy(s, r);
				b->bgp_ip = s;
				s += strlen(r) + 1;
				strcpy(s, q);
				b->bgp_name = s;
				b->bgp_next = 0;
				if(bl) bl->bgp_next=b;
				else bh = b;
				bl=b;
bcont:
				continue;
			}
			/* check to repated interface */
		        for(j=0; j < nif ; j++) {
			  if(!strncmp(ifs[j].i_name,p,strlen(ifs[j].i_name)) &&
				strlen(ifs[j].i_name) == strlen(p) ) {
				fprintf(stderr, "%s: ignore repeated object %s line %d\n",
					 gwn, ifs[j].i_name, nl);
				goto cont;
			  }	
			}
			if(fgw) {
			  for (j = 0; j < g->g_nifs; j++) {
			    ii = &(g->g_ifs[j]); 
			    if(!strncmp(ii->i_name,p,strlen(ii->i_name)) &&
				strlen(ii->i_name) == strlen(p) ) {
				fprintf(stderr, "%s: ignore repeated object %s line %d\n",
					 gwn, ii->i_name,nl);
				goto cont;
			    }
			  }
			}

			/* Save the strings */
			s = (char *) malloc(strlen(p) + strlen(r) +
			    strlen(q) + 3);
			if( s == 0 )
				goto nomem;
			if( nif >= MAXIFS ) {
				s = "too many interfaces per gateway";
				goto synterr;
			}
			strcpy(s, p);
			ifs[nif].i_name = s;
			s += strlen(p) + 1;
			strcpy(s, r);
			ifs[nif].i_logdir = s;
			s += strlen(r) + 1;
			strcpy(s, q);
			ifs[nif].i_comment = s;
			ifs[nif].i_maxinutil = ee;
			ifs[nif].i_maxoututil = ee1;
			ifs[nif].i_icmode = l_icmode;
			if(m_key == -1) 
				m_key=READ_IFDESCR;
			ifs[nif].i_map.m_key=m_key;
			ifs[nif].i_map.m_val=m_val;
			nif++;
cont:
			continue;
		}

		/* Is this a gateway declaration? */
		if( q = index(p, '{') ) {
			if( q[1] ) {
				s = "garbage after {";
				goto synterr;
			}
			/* Extract gw name */
			q--;
			while( q >= p && (*q == ' ' || *q == '\t') )
				*q-- = 0; /* It seems very strange - id can't begin with '0' */

			if( *p == 0 ) {
				s = "missing gateway name";
				goto synterr;
			}

			/* if this gateway exists already  */
			for(fgw=gwlist;fgw;fgw=fgw->g_next) {
				if (fgw->g_name && strlen(p) == strlen(fgw->g_name)
				    &&  !strncmp(fgw->g_name,p,strlen(p))) {
					gwn=fgw->g_name;
					nif=0;
					break;
				}
			}
			if(fgw) continue;    /* found existed gateway */

			q = p;		     /* community */
			SKIP_PRINTABLE(q);
			SKIP_SPACES(q);


                        r = q;               /* it may be router-id or utilization stuff */
                        SKIP_PRINTABLE(r);
                        SKIP_SPACES(r);

                        while( STREQ(r,"MINMEM_P")
                            || STREQ(r,"MINMEM_I")
                            || STREQ(r,"MINMEM_V")
                            || STREQ(r,"MINMEM_F")
                            || STREQ(r,"MINMEM_U") )
                        {
                          char *v; 

                          v = r;              /* threshold */
                          SKIP_PRINTABLE(v);
                          SKIP_SPACES(v);

                          if( !strcmp (r, "MINMEM_P" )) {
                            i = atoi(v);
                            if( i < 1 ) {
                              s = "router illegal processor memory utilization threshold";
                              goto synterr;
                            }
                            l_minmem_p=i;
                            goto nmem;
                          }
                          if( !strcmp (r, "MINMEM_V")) {
                            i = atoi(v);
                            if( i < 1 ) {
                              s = "router illegal pci memory utilization threshold";
                              goto synterr;
                            }
                            l_minmem_v=i;
                            goto nmem;
                          }
                          if( !strcmp (r, "MINMEM_I")) { 
			    i = atoi(v);
                            if( i < 1 ) {
                              s = "router illegal i/o memory utilization threshold";
                              goto synterr;
                            }
                            l_minmem_i=i;
                            goto nmem;
                          }
                          if( !strcmp (r, "MINMEM_F")) {
                            i = atoi(v);
                            if( i < 1 ) {
                              s = "router illegal fast memory utilization threshold";
                              goto synterr;
                            }
                            l_minmem_f=i;
                            goto nmem;
                          }
                          if( !strcmp (r, "MINMEM_U")) {
                            i = atoi(v);
                            if( i < 1 ) {
                              s = "router illegal multibus memory utilization threshold";
                              goto synterr;
                            }
                            l_minmem_u=i;
                            goto nmem;
                          }
nmem:
                          r = v; 
                          SKIP_PRINTABLE(r);
                          SKIP_SPACES(r);
                        }

			/* Allocate memory for gateway name */
			if(*r == 0)
				s = (char *) malloc(strlen(p) + strlen(q) + 2);
			else
				s = (char *) malloc(strlen(p) + strlen(q) + strlen(r) + 3);

			if( s == 0 )
				goto nomem;

			strcpy(s, p);
			gwn = s;
			Ngw++;
			s += strlen(p) + 1;
			strcpy(s, q);
			cmn = s;
			if (*r != 0 ) {
				s += strlen(q) + 1;
				strcpy(s,r);
				rid = s;
			}
			else rid=NULL;

			nif = 0;
			poll = 0;
			bl   = 0;
			bh   = 0;
			bgpall = 0;
			continue;
		}

		/* Variable assignment? */
		q = p;
		SKIP_PRINTABLE(q);
		if( *q == 0 ) {
			s = "missing argument";
			goto synterr;
		}
		SKIP_SPACES(q);
		/* Is it WWW-specific variable? */
		if ( !strncmp(p,"WWW",3) )
			continue;
		if( !strcmp(p, "POLL") ) {
			if( (i = rdtime(q)) < 1 ) {
				s = "bad interval specification";
				goto synterr;
			}
			if( pp == 0 )
				pp = i;
			continue;
		}
		if( !strcmp(p, "AGGREGATE") || !strcmp(p, "AGGR") ) {
			if( (i = rdtime(q)) < 1 ) {
				s = "bad interval specification";
				goto synterr;
			}
			if( ap == 0 )
				ap = i;
			continue;
		}
		if( !strcmp(p, "DIRECTORY") || !strcmp(p, "DIR") ) {
			strncpy(base_dir, q, sizeof base_dir - 1);
			continue;
		}
		if( !strcmp(p, "NOTIFYFILE") || !strcmp(p, "NOTIFY") ) {
			strncpy(notify_file, q, sizeof notify_file - 1);
			continue;
		}
		if( !strcmp(p, "LOCKFILE") || !strcmp(p, "LOCK") ) {
			strncpy(lock_file, q, sizeof lock_file - 1);
			continue;
		}
		if( !strcmp(p, "MAXERRORS") || !strcmp(p, "MAXERR") ) {
			ee = atof(q);
			if( ee < 0.0 || ee > 100.0 ) {
				s = "error rate threshold is not from 0.0 to 100.0";
				goto synterr;
			}
			err_threshold = ee;
			continue;
		}
		if( !strcmp(p, "MAXDISCARDS") || !strcmp(p, "MAXDISC")|| !strcmp(p, "MAXDROPS")) {
			ee = atof(q);
			if( ee < 0.0 || ee > 100.0 ) {
				s = "drop threshold is not from 0.0 to 100.0";
				goto synterr;
			}
			drop_threshold = ee;
			continue;
		}
		if( !strcmp(p, "MAXINUTIL")) {
			ee = atof(q);
			if( ee < 0.0 || ee > 200.0 ) {
				s = "input utilization threshold is not from 0.0 to 200.0";
				goto synterr;
			}
			inutil_threshold = ee;
			continue;
		}
		if( !strcmp(p, "MAXOUTUTIL")) {
			ee = atof(q);
			if( ee < 0.0 || ee > 200.0 ) {
				s = "output utilization threshold is not from 0.0 to 200.0";
				goto synterr;
			}
			oututil_threshold = ee;
			continue;
		}
		if( !strcmp(p, "MAXUTIL")) {
			ee = atof(q);
			if( ee < 0.0 || ee > 200.0 ) {
				s = "utilization threshold is not from 0.0 to 200.0";
				goto synterr;
			}
			inutil_threshold = ee;
			oututil_threshold = ee;
			continue;
		}
		if( !strcmp(p, "MINMEM_P")) {
			i = atoi(q);
			if( i < 1) {
			 	s = "illegal MINMEM_P threshold";
				goto synterr;
			}
			gw_minmem_p=i;
			continue;
                }
		if( !strcmp(p, "MINMEM_I")) {
			i = atoi(q);
                        if( i < 1) {
                                s = "illegal MINMEM_I threshold";
                                goto synterr;
                        }
                        gw_minmem_i=i;
                        continue;

		}
		if( !strcmp(p, "MINMEM_V")) {
		  	i = atoi(q);
                        if( i < 1) {
                                s = "illegal MINMEM_V threshold";
                                goto synterr;
                        }
                        gw_minmem_v=i;
                        continue;

		}
		if( !strcmp(p, "MINMEM_F")) {
			i = atoi(q);
                        if( i < 1) {
                                s = "illegal MINMEM_F threshold";
                                goto synterr;
                        }
                        gw_minmem_f=i;
                        continue;

		}
		if( !strcmp(p, "MINMEM_U")) {
			i = atoi(q);
                        if( i < 1) {
                                s = "illegal MINMEM_U threshold";
                                goto synterr;
                        }
                        gw_minmem_u=i;
                        continue;

		}
		if( !strcmp(p, "MAXTEMPERATURE") || !strcmp(p, "MAXTEMP") ) {
			i = atoi(q);
			if( i < 10 || i > 150 ) {
				s = "illegal temperature threshold";
				goto synterr;
			}
			gw_maxtemp = i;
			continue;
		}
		if( !strcmp(p, "MAXCPU") || !strcmp(p, "MAXCPULOAD") ) {
			i = atoi(q);
			if( i < 1 || i > 100 ) {
				s = "illegal CPU load threshold";
				goto synterr;
			}
			gw_maxcpu = i;
			continue;
		}
		if( !strcmp(p, "MINMEM") || !strcmp(p, "MINFREEMEMORY") ) {
			i = atoi(q);
			if( i < 1 ) {
				s = "illegal memory utilization threshold";
				goto synterr;
			}
			gw_minmem = i;
			continue;
		}
		if( !strcmp(p, "FILECHANGE") || !strcmp(p, "FCH") ) {
			enum FILE_CHANGE fc = rdfc(q);

			if( (int)fc == -1 ) {
				s = "illegal file change period";
				goto synterr;
			}
			file_change = fc;
			continue;
		}
		if( !strcmp(p, "STATEFILE") || !strcmp(p, "STATE") ) {
			strncpy(state_file, q, sizeof state_file - 1);
			continue;
		}
		if ( !strcmp(p, "IFSUMFILE") || !strcmp(p, "IFSUM") ) {
			strncpy(ifsum_file,q,sizeof ifsum_file - 1);
			continue;
		}
		if ( !strcmp(p, "DEBUGFILE") || !strcmp(p, "DEBUG") ) {
			strncpy(debug_file,q,sizeof debug_file - 1);
			continue;
		}
		if( !strcmp (p, "CHECK") ) {
			gw_icmode = IC_INACTIVE;
			if ( q[0] == 'a') gw_icmode = IC_INACTIVE_ATM;
			continue;
		};
		if( !strcmp(p, "FULLDOMAIN") || !strcmp(p, "FDOM") ) {
			if( *q == 'y' || *q == 'Y' )
				fulldomain = 1;
			else if( *q == 'n' || *q == 'N' )
				fulldomain = 0;
			else {
				s = "should be 'yes' or 'no'";
				goto synterr;
			}
			continue;
		}

		/* Bad syntax */
		s = "illegal statement";
		goto synterr;
	}
	if( gwn == 0 ) {
		/* Check timer intervals */
		if( pp == 0 )
			pp = DFLT_POLL;
		if( ap == 0 )
			ap = DFLT_AGGR;
		if( pp > ap ) {
			fprintf(stderr, "%s: polling period (%ds) is longer than aggregation period (%ds)\n",
			    av0, pp, ap);
			fclose(f);
			return 1;
		}
		poll_period = pp;
		aggr_period = ap;
		ngateways = Ngw;
		return 0;
	}
	s = "missing }";

synterr:
	fprintf(stderr, "%s: syntax error in %s, line %d: %s\n", av0, cfn, nl, s);
	fclose(f);
	return 1;

nomem:
	fprintf(stderr, "%s: not enough memory to read in config file %s\n", av0, cfn);
	fclose(f);
	return 1;
}

/*
 * Parse time specification
 */
int
rdtime(s)
char *s;
{
	int n = 0;
	static struct tf {
		char *str;
		int   scale;
	} tf[] = {
		"seconds",      1,
		"minutes",      60,
		"hours",        3600,
		"days",         86400,
		0,              0
	};
	struct tf *t;
	int ql;

	while( '0' <= *s && *s <= '9' )
		n = n*10 + (*s++ - '0');
	ql = strlen(s);
	if( ql == 0 )
		return n;       /* default is seconds */
	for( t = tf ; t->str ; t++ ) {
		if( !strncmp(t->str, s, ql) )
			return n * t->scale;
	}
	return -1;
}

/*
 * Read file change period
 */
enum FILE_CHANGE
rdfc(s)
char *s;
{
	static struct tf {
		char            *str;
		enum FILE_CHANGE fc;
	} tf[] = {
		"none",         NONE,
		"daily",        DAILY,
		"day",          DAILY,
		"weekly",       WEEKLY,
		"monthly",      MONTHLY,
		"yearly",       YEARLY,
		"annual",       YEARLY,
		0,              0
	};
	struct tf *t;
	int ql;

	ql = strlen(s);
	if( ql != 0 ) {
		for( t = tf ; t->str ; t++ ) {
			if( !strncasecmp(t->str, s, ql) )
				return t->fc;
		}
	}
	return (enum FILE_CHANGE) -1;
}
