/*
 * Copyright (C) 1993, US Sprint
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of US Sprint not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * US SPRINT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * US SPRINT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Author: Vadim Antonov
 *
 * $Log: rdlog.c,v $
 * Revision 1.1.1.1  1999/07/19 18:38:32  alex
 * First revision in cvs
 *
 * Revision 1.9  1999/05/11 15:24:01  alex
 * *** empty log message ***
 *
 * Revision 1.8  1998/03/20  19:02:40  alex
 * RELCOM
 *
 * Revision 1.7  1996/01/10  15:32:19  tim
 * New log format (old one is also supported)
 *
 * Revision 1.6  1996/01/05  12:10:53  tim
 * before adding drops
 *
 * Revision 1.5  1995/12/11  14:56:22  tim
 * *** empty log message ***
 *
 * Revision 1.4  1995/08/04  14:25:02  tim
 * Added some output (avg line load etc.)
 *
 * Revision 1.4  1995/08/04  14:25:02  tim
 * Added some output (avg line load etc.)
 *
 * Revision 1.3  1993/09/22  23:25:53  avg
 * Added reading router stat records.
 *
 * Revision 1.2  1993/08/11  19:40:10  avg
 * Added copyright.
 * Avoid reading floating numbers from stat records which
 * are out of time interval.
 *
 * Revision 1.2  1993/08/11  19:40:10  avg
 * Added copyright.
 * Avoid reading floating numbers from stat records which
 * are out of time interval.
 *
 * Revision 1.1  1993/08/10  17:47:57  avg
 * Initial revision
 *
 */
#ifndef lint
static char rcsid[] = "$Id: rdlog.c,v 1.1.1.1 1999/07/19 18:38:32 alex Exp $";
#endif

#include "manifest.h"

static char name[512];  /* the file name */

/*
 * Open the link log file for reading
 */
FILE *
open_linklog(lld, yr, mo, dy)
	char *lld;
	int yr;
	int mo;
	int dy;
{
	char *bbd;
	FILE *f;

	bbd = base_dir;
	if( lld[0] == 0 ) {
		bbd = "/";
		lld++;
	}
	sprintf(name, "%s/%s/%4d.%02d.%02d", bbd, lld, yr, mo, dy);
	if( f = fopen(name, "r") )
		return f;
	sprintf(name, "%s/%s/%4d.%02d", bbd, lld, yr, mo);
	if( f = fopen(name, "r") )
		return f;
	sprintf(name, "%s/%s/%4d", bbd, lld, yr);
	return fopen(name, "r");
}

u_long down_at;

/*
 * Read the link log file
 * Returns -1 on end of file, -2 on "got all records" and IF_somth for
 * various types of records.
 */
int
rdlog(f, tt, la, speedp, start, end)
	FILE *f;
	u_long *tt;
	struct LAGG *la;
	double *speedp;
	u_long start, end;
{
	char line[500];
	char *bp;
	static tini;
	static struct tm tm;

	if( tini == 0 ) {
		tini++;
		tm = *localtime(&start);
	}

	/* Read in the line */
next:   if( fgets(line, sizeof line, f) == 0 )
		return -1;

	/* Parse timestamp */
	bp = line;
	if( (tm.tm_year = iget(&bp)) < 0 || *bp++ != '.' ||
	    (tm.tm_mon  = iget(&bp)) < 0 || *bp++ != '.' ||
	    (tm.tm_mday = iget(&bp)) < 0 || *bp++ != ' ' ||
	    (tm.tm_hour = iget(&bp)) < 0 || *bp++ != ':' ||
	    (tm.tm_min  = iget(&bp)) < 0 || *bp++ != ':' ||
	    (tm.tm_sec  = iget(&bp)) < 0 || *bp != ' ' ) {
		fprintf(stderr, "%s: corrupted timestamp in log file %s\n\t%s\n", av0, name, line);
		goto next;
	}
	if( tm.tm_mon < 1 || tm.tm_mon > 12 || tm.tm_mday > 31 ||
	    tm.tm_hour > 23 || tm.tm_min > 59 || tm.tm_sec > 59 ) {
		fprintf(stderr, "%s: impossible numbers in timestamp in log file %s\n\t%s\n", av0, name, line);
		goto next;
	}
	if( tm.tm_year < 80 )
		tm.tm_year += 100;
	tm.tm_mon--;
	*tt = timelocal(&tm);
	if( *tt >= end )
		return -2;

	while( *bp == ' ' )
		bp++;

	/* Check for status change records */
	if( !strncmp(bp, "UP",2) || !strncmp(bp,"FIRST",5) ) {
		if( *tt < start ) {
			down_at = (u_long) -1;
			goto next;
		}
		return IF_UP;
	}
	if(  !strncmp(bp, "DOWN",4) || !strncmp(bp, "INAC",4) ) {
		if( *tt < start ) {
			down_at = start;
			goto next;
		}
		return IF_DOWN;
	}
	if( !strncmp(bp, "TESTING",7) ) {
		if( *tt < start ) {
			down_at = start;
			goto next;
		}
		return IF_TESTING;
	}
	if( *tt < start )
		goto next;

	/* Parse link stats */
	/*           speed  inutil  inpkt   oututil  outpkt   inerr   outerr   indrop  outdrop */
	if( sscanf(bp, "%lf %lf/%lf %lf/%lf, %lf/%lf %lf/%lf, %lf/%lf %lf/%lf, %lf/%lf %lf/%lf\n",
	       speedp,
	       &la->s_inutil, &la->s_inpeakutil,
	       &la->s_inpckts, &la->s_inpeakpckts,
	       &la->s_oututil, &la->s_outpeakutil,
	       &la->s_outpckts, &la->s_outpeakpckts,
	       &la->s_inerrors, &la->s_inpeakerrors,
	       &la->s_outerrors, &la->s_outpeakerrors,
	       &la->s_indrops, &la->s_inpeakdrops,
	       &la->s_outdrops, &la->s_outpeakdrops) != 17 ) {

	       /* try to read old format if possible */
	       la->s_inpeakerrors = 0.0;
	       la->s_outerrors = la->s_outpeakerrors = 0.0;
	       la->s_indrops = la->s_inpeakdrops = 0.0;
	       la->s_outpeakdrops = 0.0;
	       if (sscanf(bp, "%lf %lf/%lf %lf/%lf, %lf/%lf %lf/%lf, %lf %lf\n",
		 speedp,
		 &la->s_inutil, &la->s_inpeakutil,
		 &la->s_inpckts, &la->s_inpeakpckts,
		 &la->s_oututil, &la->s_outpeakutil,
		 &la->s_outpckts, &la->s_outpeakpckts,
		 &la->s_inerrors, &la->s_outdrops) != 11 ) {
		    fprintf(stderr, "%s: corrupted record in log file %s\n\t%s\n", av0, name, line);
		    goto next;
		}
	}
	if( la->s_inutil > 100.0 || la->s_inpeakutil > 100.0 ||
	    la->s_oututil > 100.0 || la->s_outpeakutil > 100.0 ||
	    la->s_inerrors > 100.0 || la->s_outdrops > 100.0 ) {
		fprintf(stderr, "%s: illegal values in the log file %s\n\t%s\n", av0, name, line);
	       fprintf(stderr, "%f %f/%f %f/%f, %f/%f %f/%f, %f %f\n",
	       speedp,
	       la->s_inutil, la->s_inpeakutil,
	       la->s_inpckts, la->s_inpeakpckts,
	       la->s_oututil, la->s_outpeakutil,
	       la->s_outpckts, la->s_outpeakpckts,
	       la->s_inerrors, la->s_outdrops);
		goto next;
	}
	return IF_STAT;
}

/*
 * Read the router log file
 * Returns -1 on end of file, -2 on "got all records" and IF_somth for
 * various types of records.
 */
int
rdrlog(f, tt, gs, start, end)
	FILE *f;
	u_long *tt;
	struct GAGG *gs;
	u_long start, end;
{
	char line[256];
	char *bp;
	static tini;
	static struct tm tm;

	if( tini == 0 ) {
		tini++;
		tm = *localtime(&start);
	}

	/* Read in the line */
next:   if( fgets(line, sizeof line, f) == 0 )
		return -1;

	/* Parse timestamp */
	bp = line;
	if( (tm.tm_year = iget(&bp)) < 0 || *bp++ != '.' ||
	    (tm.tm_mon  = iget(&bp)) < 0 || *bp++ != '.' ||
	    (tm.tm_mday = iget(&bp)) < 0 || *bp++ != ' ' ||
	    (tm.tm_hour = iget(&bp)) < 0 || *bp++ != ':' ||
	    (tm.tm_min  = iget(&bp)) < 0 || *bp++ != ':' ||
	    (tm.tm_sec  = iget(&bp)) < 0 || *bp != ' ' ) {
		fprintf(stderr, "%s: corrupted timestamp in log file %s\n\t%s\n", av0, name, line);
		goto next;
	}
	if( tm.tm_mon < 1 || tm.tm_mon > 12 || tm.tm_mday > 31 ||
	    tm.tm_hour > 23 || tm.tm_min > 59 || tm.tm_sec > 59 ) {
		fprintf(stderr, "%s: impossible numbers in timestamp in log file %s\n\t%s\n", av0, name, line);
		goto next;
	}
	if( tm.tm_year < 80 )
		tm.tm_year += 100;
	tm.tm_mon--;
	*tt = timelocal(&tm);
	if( *tt >= end )
		return -2;

	while( *bp == ' ' )
		bp++;

	/* Check for status change records */
	if( !strncmp(bp, "UP",2) || !strncmp(bp,"FIRST",5) ) {
		long ttl;
		if( *tt < start ) {
			down_at = (u_long) -1;
			goto next;
		}
		/* We have 'time_stamp UP time_of_live\n'
		 * if time_of_live > time_of_down. then
		 * it was not router failure but it was router unavailability
		 * If so, we reset down time
		 */
		bp += 2 ;
		while( *bp == ' ' )
			bp++;
		if ( *bp && *bp != '\n' && (ttl = iget(&bp)) > 0 && down_at != (u_long)(-1) )
		{
			if ( down_at + ttl > *tt ) down_at = (u_long)(-1);
		}
		return IF_UP;
	}
	if( !strncmp(bp, "INAC",4) || !strncmp(bp, "DOWN",4) ) {
		if( *tt < start ) {
			down_at = start;
			goto next;
		}
		return IF_DOWN;
	}
	if( *tt < start )
		goto next;

	/* Parse router stats */
	if( sscanf(bp, "%u/%u %lu/%lu %d",
			&gs->gs_cpuavg, &gs->gs_cpumax,
			&gs->gs_memavg, &gs->gs_memmin,
			&gs->gs_temp) != 5 ) {
		fprintf(stderr, "%s: corrupted record in log file %s\n\t%s\n", av0, name, line);
		goto next;
	}
	if( gs->gs_cpuavg > gs->gs_cpumax || gs->gs_cpumax > 100 ||
	    gs->gs_memmin > gs->gs_memavg || gs->gs_memavg < 0 ||
	    gs->gs_memmin <= 0.0 || gs->gs_temp > 100 ) {
		fprintf(stderr, "%s: illegal values in the log file %s\n\t%s\n", av0, name, line);
		fprintf(stderr, "%d/%d %lu/%lu %d\n",
				gs->gs_cpuavg, gs->gs_cpumax,
				gs->gs_memavg, gs->gs_memmin,
				gs->gs_temp);
		goto next;
	}
	return IF_STAT;
}

/*
 * Yet another variation on a theme of atoi
 */
int
iget(p)
	char **p;
{
	int n = 0;
	register c;
	char *q = *p;

	while( (c = *q - '0') >= 0 && c <= 9 ) {
		n = n*10 + c;
		q++;
	}
	if( *p == q )
		return -1;
	*p = q;
	return n;
}
