/*
 * Copyright (C) 1993, US Sprint
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of US Sprint not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * US SPRINT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * US SPRINT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Author: Vadim Antonov
 *
 * $Log: sched.c,v $
 * Revision 1.1.1.1  1999/07/19 18:38:32  alex
 * First revision in cvs
 *
 * Revision 1.5  1998/03/20  19:02:40  alex
 * RELCOM
 *
 * Revision 1.4  1995/12/11  14:56:22  tim
 * *** empty log message ***
 *
 * Revision 1.4  1995/12/11  14:56:22  tim
 * *** empty log message ***
 *
 * Revision 1.3  1995/08/04  14:25:02  tim
 * Added some output (avg line load etc.)
 *
 * Revision 1.3  1995/08/04  14:25:02  tim
 * Added some output (avg line load etc.)
 *
 * Revision 1.2  1993/09/22  23:25:53  avg
 * *** empty log message ***
 *
 * Revision 1.1  1993/08/10  17:47:57  avg
 * Initial revision
 *
 * Revision 1.1  1993/08/10  17:47:57  avg
 * Initial revision
 *
 */
#ifndef lint
static char rcsid[] = "$Id: sched.c,v 1.1.1.1 1999/07/19 18:38:32 alex Exp $";
#endif


#include "manifest.h"

struct SNMP_ACTION *aqh;     /* Action queue head */

/*
 * The scheduler -- if action is immediate, do it.
 * Returns the delay (in seconds) before the next action.
 */
int
sched()
{
	u_long t;
	struct SNMP_ACTION *a;

	while( aqh != 0 ) {
		t = time(0);
		if( aqh->a_time > t )
			return aqh->a_time - t;
		a = aqh;
		aqh = a->a_next;
		(*(a->a_f)) (a);
	}
	return 1;              /* no actions scheduled */
}

/*
 * Add an action to be done at the given time to the queue
 */
void
timeout(a)
	struct SNMP_ACTION *a;
{
	struct SNMP_ACTION *p, *po;

	/* Drop old _a_ record from queue */
	if ( aqh == a ) aqh = aqh->a_next;
	for ( p = aqh ; p ; p = p->a_next )
	       if ( p ->a_next == a ) {
		       p->a_next = a->a_next;
	       };
	/* Found the best place */
	po = 0;
	for( p = aqh ; p ; p = p->a_next ) {
		if( p->a_time > a->a_time )
			break;
		po = p;
	}
	if( po == 0 ) { /* the first in q */
		a->a_next = aqh;
		aqh = a;
	} else {
		a->a_next = po->a_next;
		po->a_next = a;
	}
}

/*
 * Shift the time tt to be sharp at (12am + N*period),
 * local time.
 * Makes log files cute.
 */
u_long
tsharp(tt, period, off)
	u_long tt;
	u_long period;
	int off;
{
	struct tm *tm;
	u_long defect;

	if( period > (12*60*60) )
		return tt;
	tm = localtime(&tt);
	defect = tm->tm_sec + 60 * tm->tm_min + 3600 * tm->tm_hour + off;
	defect %= period;
	return tt + period - defect;
}
