/*
 * Copyright (C) 1993, US Sprint
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of US Sprint not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * US SPRINT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * US SPRINT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Author: Vadim Antonov
 *
 */
#ifndef lint
static char rcsid[] = "$Id: snmptbl.c,v 1.1.1.1 1999/07/19 18:38:32 alex Exp $";
#endif

#include "manifest.h"

/*
 * Report the troubles in the network
 */
#include <ctype.h>
#include <math.h>

#ifndef REGEXPINCL
#define INIT    register char *bp = instring;
#define GETC()  (*bp++)
#define PEEKC() (*bp)
#define UNGETC(c) bp--
#define RETURN(c) return (c);
#define ERROR(c) regerr(c)
#include <regexp.h>
#else
#include REGEXPINCL
#endif

char *av0;

char expbuf[1024];

static char *mon_names[] = {
	"January", "February", "March", "April",
	"May", "June", "July", "August",
	"September", "October", "November", "December"
};

char *numeral();

int parsedate();

char *arg_pp, *arg_ap;

FILE *open_linklog();

char *tod(), *saytime();

extern u_long down_at;

main(ac, av)
	int ac;
	char **av;
{
	extern char *optarg;
	extern int optind;
	FILE *prolog;
	register char c;
	char *cf = CONFIG_FILE;

	char *date;
	int yr, mo, dy; /* starting year/month/day */
	int cyr, cmo, cdy, hr;
	int i, ndays, nmatch, ntroubled;
	int nrtroubled, nrmatch;

#ifndef REGEXPINCL
	char *res[128]; /* Regular expressions */
#else
	regexp *res[128];
#endif
	char *re, *index(), *p;
	int nres, rn;
	char buf[512];

	enum FILE_CHANGE period = NONE;
	enum FILE_CHANGE rfc();

	struct GATEWAY *g;
	struct IF *ii;
	FILE *f;

	struct LAGG la;
	struct GAGG ga;
	char   *i_pname;
	struct tm tm;
	u_long tt, start_tt, end_tt;
	int hr_start, hr_end;


	char *m_ln;
	struct GATEWAY *ok_g, *m_g;
	double speed;
	double avg_inerrs, avg_outdrops;
	double avg_inutil, avg_oututil;
	double avg_inpckts, avg_outpckts;
	int n_inoverhead, n_outoverhead;
	int n_stat, n_inerrs, n_outdrops, n_recs;
	int n_down;
	u_long total_down;
	int ndt;
	double tot;

	int r_cnt, r_maxtemp, r_n_temp, r_avgcpu, r_n_cpu, r_avgmem, r_n_mem;

	int mtbf = 0;
	int ever_up;    /* link/router was ever up */
	long last_fail;  /* last failure ended at */
	long l_up_sum, l_down_sum;
	int l_mean_ctr;
	long r_up_sum, r_down_sum;
	int r_mean_ctr;
	double data_total;

	av0 = av[0];
	hr_start = 0;
	hr_end = 24;
	for(;;) {
		switch( getopt(ac, av, "mc:h:t:") ) {
		case -1:
			if( optind < ac )
				break;
		default:
		usage:
			fprintf(stderr, "Usage: %s [-m] [-c config_file] [-h hour-hour] [-t period] year.month[.day] [name...]\n", av0);
			exit(1);

		case 'c':
			cf = optarg;
			continue;

		case 'm':
			mtbf = 1;
			continue;

		case 't':
			period = rdfc(optarg);
			if( period < DAILY || period > MONTHLY ) {
				fprintf(stderr, "%s: bad period, valid periods are: day, week, month\n", av0);
				exit(1);
			}
			continue;

		case 'h':
			p = optarg;
			hr_start = iget(&p);
			if( *p++ != '-' || hr_start < 0 || hr_start > 23 )
				goto usage;
			hr_end = iget(&p);
			if( hr_end == 0 )
				hr_end = 24;
			if( *p != 0 || hr_end <= hr_start )
				goto usage;
			continue;

		}
		break;
	}

	/*
	 * Parse date
	 */
	date = av[optind++];
	if (parsedate (date, &yr, &mo, &dy, &period)) {
		fprintf(stdout, "%s: bad date format (should be like 93.2 or 2001.11.03)\n", av0);
		exit(1);
	}
	/*
	 * Compile regular expressions
	 */
#ifndef REGEXPINCL
	re = expbuf;
	nres = 0;
	for( i = optind ; i < ac ; i++ ) {
		if( nres >= sizeof res / sizeof res[0] ) {
			fprintf(stderr, "%s: too many regular expressions\n", av[0]);
			exit(1);
		}
		res[nres++] = re;
		for( p = av[i] ; *p ; p++ ) {
			if( isupper(*p) )
				*p = tolower(*p);
		}
		re = compile(av[i], re, &expbuf[sizeof expbuf - 1], 0);
	}
#else
	nres = 0;
	for( i = optind ; i < ac ; i++ ) {
		for( p = av[i] ; *p ; p++ ) {
			if( isupper(*p) )
				*p = tolower(*p);
		}
		if( (res[nres++] = regcomp(av[i])) == 0 )
			exit(1);
	}
#endif /*REGEXPINCL*/


	/*
	 * Read in the configuration file
	 */
	if( rdconfig(cf) )
		exit(1);

	/*
	 * Generate timestamp records
	 */
	switch( period ) {
		case MONTHLY:
			hr_start = 0;
			hr_end = 24;
			ndays = mondays (mo, yr);
			dy = 1;
			break;
		case WEEKLY:
			hr_start = 0;
			hr_end = 24;
			ndays = 7;
			break;
		case DAILY:
			ndays = 1;
			break;
	}


	/*
	 * Calculate the start time
	 */
	tt = time(0);
	tm = *localtime(&tt);
	tm.tm_year = yr - 1900;
	tm.tm_mday = dy;
	tm.tm_mon = mo - 1;
	tm.tm_hour = 0;
	tm.tm_min = 0;
	tm.tm_sec = 0;
	start_tt = timelocal(&tm);
	end_tt = start_tt + ndays * 24 * 60 * 60;
	start_tt += hr_start * 60 * 60;
	end_tt -= (24 - hr_end) * 60 * 60;
	tot = end_tt - start_tt;

	/*
	 * Print the header
	 */
	printf("<H1>Network Reliability Report '%d/", mo);
	if( period != MONTHLY )
		printf("%d/", dy);
	printf("%d'\n", yr);
	printf("</H1>\n<H4>ROUTERS  RELIABLITY  REPORT ");
	time_header (tt, start_tt, end_tt, yr, mo, dy, hr_start, hr_end, period);
	printf("</H4>\n");

	if( end_tt > tt )
		end_tt = tt;

	/*
	 * Initialize MTBF & MTBR counters
	 */
	l_up_sum = l_down_sum = 0;
	r_up_sum = r_down_sum = 0;
	r_mean_ctr = l_mean_ctr = 0;
	data_total = 0.0;

	/*
	 * Scan all the routers matching the pattern
	 */
	nrmatch = 0;
	nrtroubled = 0;

	/* output table heading */

	printf("<TABLE BORDER=1 WIDTH=\"100%%\">");
	printf("<TR><TH ALIGN=CENTER>Router</TH><TH ALIGN=CENTER COLSPAN=2>Down</TH><TH ALIGN=CENTER COLSPAN=2>CPU load</TH><TH ALIGN=CENTER COLSPAN=2>Memory</TH><TH ALIGN=CENTER>UP</TH></TR>\n");
	printf("<TR><TH ALIGN=CENTER>name</TH><TH ALIGN=CENTER>num</TH><TH>time<BR>min</TH><TH>avg</TH><TH>high<BR>min</TH><TH>avg</TH><TH>low<BR>min</TH><TH>time</TH></TR>\n");
	for( rn = (nres?0:-1) ; rn < nres ; rn++ ) {
		for( g = gwlist ; g ; g = g->g_next ) {
			if( nres ) {
				if ( g->g_sname ) {
				    strcpy(buf, g->g_sname);
				    for( p = buf ; *p ; p++ ) {
					    if( isupper(*p) )
						    *p = tolower(*p);
				    }
#ifndef REGEXPINCL
				    if( !step(buf, res[rn]) )
					    goto next;
#else
				    if( !regexec(res[rn], buf) )
					    goto next;
#endif
				} else {
next:
				    strcpy(buf, g->g_name);
				    for( p = buf ; *p ; p++ ) {
					    if( isupper(*p) )
						    *p = tolower(*p);
				    }
#ifndef REGEXPINCL
				    if( !step(buf, res[rn]) )
					    continue;
#else
				    if( !regexec(res[rn], buf) )
					    continue;
#endif
				};
			}

			/* Open the router log file */
			cyr = yr; cmo = mo; cdy = dy;
			f = open_linklog(g->g_name, yr, mo, dy);
			if( f == 0 ) {
				if (strcmp(g->g_name, g->g_sname))
					fprintf(stderr, "%s(%s): (warning) -- no log file %s/%d.%02d.%02d\n",
						av0, g->g_sname, g->g_name, yr, mo, dy);
				else
					fprintf(stderr, "%s: (warning) -- no log file %s/%d.%02d.%02d\n",
						av0, g->g_name, yr, mo, dy);
				continue;
			}
			/* Read the records */
			nrmatch++;
			down_at = (u_long)-1;
			n_down = 0;
			n_recs = 0;
			total_down = 0;
			r_cnt = 0;
			r_maxtemp = 0;
			r_n_temp = 0;
			r_avgcpu = 0;
			r_n_cpu = 0;
			r_avgmem = 0;
			r_n_mem = 0;
			last_fail = (u_long) -1;
			ever_up = 0;
		rcont:  while( (i = rdrlog(f, &tt, &ga, start_tt, end_tt)) > 0 ) {
				/*
				 * If we're printing the link listings
				 * show the link.
				 */
				n_recs++;
				/* Process records */
				switch( i ) {
				case IF_UP:
					ever_up = 1;
					if( down_at == (u_long)-1 )
					       break;
					i = tt - down_at;
					if( i > 1 ) {
						n_down++;
						total_down += i;
						if( i > 120 && ever_up ) {
							r_mean_ctr++;
							r_down_sum += i;
							if( last_fail == (u_long)-1 )
								r_up_sum += down_at - start_tt;
							else
								r_up_sum += down_at - last_fail;
							last_fail = tt;
						}
					}
					down_at = (u_long)-1;
					break;

				case IF_DOWN:
					if( down_at == (u_long)-1 )
						down_at = tt;
					break;

				case IF_STAT:
					r_cnt++;
					if( ga.gs_temp > r_maxtemp )
						r_maxtemp = ga.gs_temp;
					if( ga.gs_temp > gw_maxtemp )
						r_n_temp++;
					r_avgcpu += ga.gs_cpuavg;
					if( ga.gs_cpuavg > gw_maxcpu )
						r_n_cpu++;
					r_avgmem += ga.gs_memavg;
					if( ga.gs_memavg < gw_minmem )
						r_n_mem++;
					break;
				}
			}
			if( i == -2 )
				goto rend_reached;

			/* Try to open the next log file */
			switch( file_change ) {
			case DAILY:
				if( period == DAILY )
					break;
			rnextday:
				if( ++cdy > mondays (cmo, cyr) ) {
					cdy = 1;
					if( ++cmo > 12 ) {
						cmo = 1;
						cyr++;
					}
				}

				/* Do we need to open a new file? */
				tm.tm_year = cyr - 1900;
				tm.tm_mday = cdy;
				tm.tm_mon = cmo - 1;
				tm.tm_hour = 0;
				tm.tm_min = 0;
				if( timelocal(&tm) >= end_tt )
					break;

				/* Try to open the new log file */
				fclose(f);
				f = open_linklog(g->g_name, cyr, cmo, cdy);
				if( f != 0 )
					goto rcont;
				break;

			case WEEKLY:
				cdy += 6;
				goto rnextday;

			case MONTHLY:
				if( period == DAILY || period == MONTHLY )
					break;
				cdy += 100;
				goto rnextday;
			}

		rend_reached:
			if( down_at != (u_long)-1 ) {
				n_down++;
				total_down += end_tt - down_at;
			} else {
				if( last_fail == (u_long)-1 )
					r_up_sum += end_tt - start_tt;
				else
					r_up_sum += end_tt - last_fail;
			}

			if (!r_cnt) /* ???? */
			    r_cnt = 1;

			printf("<TR><TD>%s</TD><TD>%d</TD><TD>%d</TD><TD>%d</TD><TD>%d</TD><TD>%.2f</TD><TD>%d</TD><TD>%.2f</TD></TR>\n", g->g_sname,
				n_down, total_down / 60,
				r_avgcpu/r_cnt, (aggr_period * r_n_cpu) / 60,
				(r_avgmem/1024.0)/r_cnt, (aggr_period * r_n_mem) / 60,
				100.0* (tot - total_down) / tot );
			if( n_recs == 0 ) {
				nrmatch--;
				fprintf(stderr,"%s: NO DATA AVAILABLE\n", g->g_name);
			}
			if( f != 0 )
				fclose(f);
		}

	}
	printf("</TABLE>\n");

	/*
	 * Scan all the links matching the pattern
	 */
	nmatch = 0;
	ntroubled = 0;
	tt = time(0);

	/* output table heading */
	printf("<H4>CHANNEL  UTILISATION  REPORT ");
	time_header (tt, start_tt, end_tt, yr, mo, dy, hr_start, hr_end, period);
	printf("</H4>\n");

	printf("<TABLE BORDER=1 WIDTH=\"100%%\">\n");
	printf("<TR><TH>Link</TH><TH>Speed</TH><TH COLSPAN=4>Average Utilisation</TH><TH COLSPAN=2>High Load</TH></TR>\n");
	printf("<TR><TH>name</TH><TH>Kbps</TH><TH COLSPAN=2>Input</TH><TH COLSPAN=2>Output</TH><TH>in</TH><TH>out</TH></TR>\n");
	printf("<TR><TH COLSPAN=2></TH></TH><TH>%%</TH><TH>pkt/s</TH><TH>%%</TH><TH>pkt/s</TH><TH>min</TH><TH>min</TH></TR>\n");
	for( rn = (nres?0:-1) ; rn < nres ; rn++ ) {
		for( g = gwlist ; g ; g = g->g_next ) {
			for( ii = g->g_ifs ; ii < &g->g_ifs[g->g_nifs] ; ii++ ) {
				if( nres == 0 )
					goto matched;

				/* Compare for full.router.name:interface  */
				strcpy(buf, g->g_name);
				strcat(buf, ":");
				strcat(buf, ii->i_name);
				if( !strcasecmp(av[rn+optind], buf) )
					goto matched;

				/* Compare for short-router-name:interface */
				strcpy(buf, g->g_sname);
				strcat(buf, ":");
				strcat(buf, ii->i_name);
				if( !strcasecmp(av[rn+optind], buf) )
					goto matched;
/* deleted by tim@sssun
				if( re = index(buf, '.') ) {
					*re = ':';
					strcpy(re+1, ii->i_name);
					if( !strcasecmp(av[rn+optind], buf) )
						goto matched;
				}
*/
				/* Try to mactch file and comment fields */
				re = buf;
				for( p = ii->i_comment ; *p ; p++ )
					*re++ = isupper(*p) ? tolower(*p) : *p;
				*re = 0;
#ifndef REGEXPINCL
				if( step(buf, res[rn]) )
					goto matched;
#else
				if( regexec(res[rn], buf) )
					goto matched;
#endif
				re = buf;
				for( p = ii->i_logdir ; *p ; p++ )
					*re++ = isupper(*p) ? tolower(*p) : *p;
				*re = 0;
#ifndef REGEXPINCL
				if( step(buf, res[rn]) )
					goto matched;
#else
				if( regexec(res[rn], buf) )
					goto matched;
#endif
				continue;

			matched:
				if( ii->i_index++ )
					continue;       /* already processed! */

				/* Open the link log file */
				cyr = yr; cmo = mo; cdy = dy;
				f = open_linklog(ii->i_logdir, yr, mo, dy);
				if( f == 0 ) {
					       fprintf(stderr, "%s: (warning) -- no log file %s/%d.%02d.%02d\n",
							av0, ii->i_logdir, yr, mo, dy);
					continue;
				}

				/* Read the records */
				nmatch++;
				n_down = 0;
				total_down = 0;
				avg_inutil = 0.0;
				avg_oututil = 0.0;
				avg_inpckts = 0.0;
				avg_outpckts = 0.0;
				n_stat = 0;
				n_recs = 0;
				n_inoverhead = 0;
				n_outoverhead = 0;
			cont1:  while( (i = rdlog(f, &tt, &la, &speed, start_tt, end_tt)) > 0 ) {
					n_recs++;
					/*
					 * If we're printing the link listings
					 * show the link.
					 */

					/* Process records */
					switch( i ) {
					case IF_UP:
					case IF_DOWN:
					case IF_TESTING:
						break;

					case IF_STAT:
						n_stat++;
						avg_inutil += la.s_inutil;
						if (la.s_inutil > ii->i_maxinutil)
							n_inoverhead++;
						avg_oututil += la.s_oututil;
						if (la.s_oututil > ii->i_maxoututil)
							n_outoverhead++;
						avg_inpckts += la.s_inpckts;
						avg_outpckts += la.s_outpckts;
					}
				}
				if( i == -2 )
					goto end_reached;

				/* Try to open the next log file */
				switch( file_change ) {
				case DAILY:
					if( period == DAILY )
						break;
				nextday1:
					if( ++cdy > mondays (cmo, cyr) ) {
						cdy = 1;
						if( ++cmo > 12 ) {
							cmo = 1;
							cyr++;
						}
					}

					/* Do we need to open a new file? */
					tm.tm_year = cyr - 1900;
					tm.tm_mday = cdy;
					tm.tm_mon = cmo - 1;
					tm.tm_hour = 0;
					tm.tm_min = 0;
					tm.tm_sec = 0;
					if( timelocal(&tm) >= end_tt )
						break;

					/* Try to open the new log file */
					fclose(f);
					f = open_linklog(ii->i_logdir, cyr, cmo, cdy);
					if( f != 0 )
						goto cont1;
					break;

				case WEEKLY:
					cdy += 6;
					goto nextday1;

				case MONTHLY:
					if( period == DAILY || period == MONTHLY )
						break;
					cdy += 100;
					goto nextday1;
				}

			end_reached:
				if ( n_stat == 0 ) n_stat = 1;
				i_pname = ii->i_comment[0] ? ii->i_comment : ii->i_name;
				if (p = index (i_pname, ' '))
					*p = '\0';
				avg_inutil /= n_stat;
				if (isnan(avg_inutil))
					avg_inutil = 0.0;
				avg_oututil /= n_stat;
				if (isnan(avg_oututil))
					avg_oututil = 0.0;
				avg_inpckts /= n_stat;
				if (isnan(avg_inpckts))
					avg_inpckts = 0.0;
				avg_outpckts /= n_stat;
				if (isnan(avg_outpckts))
					avg_outpckts = 0.0;

				printf ("<TR><TD>%s</TD><TD>%.1f</TD><TD>%d</TD><TD>%d</TD><TD>%d</TD><TD>%d</TD><TD>%d</TD><TD>%d</TD></TR>\n",
                                        i_pname, speed,
                                        (int)rint(avg_inutil), (int)rint(avg_inpckts),
                                        (int)rint(avg_oututil), (int)rint (avg_outpckts),
                                        (n_inoverhead * aggr_period) / 60,
                                        (n_outoverhead * aggr_period) / 60 );


				if( n_recs == 0 ) {
					nmatch--;
					fprintf(stderr, "%s -> %s: NO DATA AVAILABLE\n", g->g_name, ii->i_comment);
				}
				if( f != 0 )
					fclose(f);
			}
		}
	}

	printf("</TABLE>\n");

	/*
	 * Scan all the links matching the pattern
	 */
	nmatch = 0;
	ntroubled = 0;
	tt = time(0);

	/* output table heading */
	printf("<H4>CHANNEL  RELIABILITY  REPORT ");
	time_header (tt, start_tt, end_tt, yr, mo, dy, hr_start, hr_end, period);
	printf("</H4>\n");

	printf("<TABLE BORDER=1 WIDTH=\"100%%\">\n");
	printf("<TR><TH>Link</TH><TH COLSPAN=2>Down</TH><TH COLSPAN=2>In Errors</TH><TH COLSPAN=2>Out Drops</TH></TR>\n");
	printf("<TR><TH>name</TH><TH>times</TH><TH>sum<BR>min</TH><TH>avg</TH><TH>high<BR>min</TH><TH>avg</TH><TH>high<BR>min</TH></TR>\n");
	for( rn = (nres?0:-1) ; rn < nres ; rn++ ) {
		for( g = gwlist ; g ; g = g->g_next ) {
			for( ii = g->g_ifs ; ii < &g->g_ifs[g->g_nifs] ; ii++ ) {
				if( nres == 0 )
					goto matched1;

				/* Compare for full.router.name:interface  */
				strcpy(buf, g->g_name);
				strcat(buf, ":");
				strcat(buf, ii->i_name);
				if( !strcasecmp(av[rn+optind], buf) )
					goto matched1;

				/* Compare for short-router-name:interface */
				strcpy(buf, g->g_sname);
				strcat(buf, ":");
				strcat(buf, ii->i_name);
				if( !strcasecmp(av[rn+optind], buf) )
					goto matched1;
				/* Try to mactch file and comment fields */
				re = buf;
				for( p = ii->i_comment ; *p ; p++ )
					*re++ = isupper(*p) ? tolower(*p) : *p;
				*re = 0;
#ifndef REGEXPINCL
				if( step(buf, res[rn]) )
					goto matched1;
#else
				if( regexec(res[rn], buf) )
					goto matched1;
#endif
				re = buf;
				for( p = ii->i_logdir ; *p ; p++ )
					*re++ = isupper(*p) ? tolower(*p) : *p;
				*re = 0;
#ifndef REGEXPINCL
				if( step(buf, res[rn]) )
					goto matched1;
#else
				if( regexec(res[rn], buf) )
					goto matched1;
#endif
				continue;

			matched1:
				if( ii->i_index++ > 1 )
					continue;       /* already processed! */

				/* Open the link log file */
				cyr = yr; cmo = mo; cdy = dy;
				f = open_linklog(ii->i_logdir, yr, mo, dy);
				if( f == 0 ) {
					       fprintf(stderr, "%s: (warning) -- no log file %s/%d.%02d.%02d\n",
							av0, ii->i_logdir, yr, mo, dy);
					continue;
				}

				/* Read the records */
				nmatch++;
				down_at = (u_long)-1;
				n_down = 0;
				total_down = 0;
				avg_inerrs = 0.0;
				avg_outdrops = 0.0;
				n_stat = 0;
				n_inerrs = 0;
				n_outdrops = 0;
				n_recs = 0;
				last_fail = (u_long) -1;
				ever_up = 0;
			cont2:  while( (i = rdlog(f, &tt, &la, &speed, start_tt, end_tt)) > 0 ) {
					n_recs++;
					/*
					 * If we're printing the link listings
					 * show the link.
					 */

					/* Process records */
					switch( i ) {
					case IF_UP:
						ever_up = 1;
						if( down_at == (u_long)-1 )
							continue;
						i = tt - down_at;
						if( i > 1 ) {
							n_down++;
							total_down += i;
							if( i > 120 && ever_up ) {
								l_mean_ctr++;
								l_down_sum += i;
								if( last_fail == (u_long)-1 )
									l_up_sum += down_at - start_tt;
								else
									l_up_sum += down_at - last_fail;
								last_fail = tt;
							}
						}
						down_at = (u_long)-1;
						break;

					case IF_DOWN:
					case IF_TESTING:
						if( down_at != (u_long)-1 )
							continue;
						down_at = tt;
						break;

					case IF_STAT:
						n_stat++;
						avg_inerrs += la.s_inerrors;
						if( la.s_inerrors > err_threshold )
							n_inerrs++;
						avg_outdrops += la.s_outdrops;
						if (la.s_outdrops > drop_threshold)
							n_outdrops++;
						data_total += (poll_period * speed / 8.0) * la.s_oututil;
					}
				}
				if( i == -2 )
					goto end_reached1;

				/* Try to open the next log file */
				switch( file_change ) {
				case DAILY:
					if( period == DAILY )
						break;
				nextday2:
					if( ++cdy > mondays (cmo, cyr) ) {
						cdy = 1;
						if( ++cmo > 12 ) {
							cmo = 1;
							cyr++;
						}
					}

					/* Do we need to open a new file? */
					tm.tm_year = cyr - 1900;
					tm.tm_mday = cdy;
					tm.tm_mon = cmo - 1;
					tm.tm_hour = 0;
					tm.tm_min = 0;
					tm.tm_sec = 0;
					if( timelocal(&tm) >= end_tt )
						break;

					/* Try to open the new log file */
					fclose(f);
					f = open_linklog(ii->i_logdir, cyr, cmo, cdy);
					if( f != 0 )
						goto cont2;
					break;

				case WEEKLY:
					cdy += 6;
					goto nextday1;

				case MONTHLY:
					if( period == DAILY || period == MONTHLY )
						break;
					cdy += 100;
					goto nextday2;
				}

			end_reached1:
				if ( n_stat == 0 ) n_stat = 1;
				if( down_at != (u_long)-1 ) {
					n_down++;
					total_down += end_tt - down_at;
				} else {
					if( last_fail == (u_long)-1 )
						l_up_sum += end_tt - start_tt;
					else
						l_up_sum += end_tt - last_fail;
				}
				i_pname = ii->i_comment[0] ? ii->i_comment : ii->i_name;
				if (p = index (i_pname, ' '))
					*p = '\0';
				avg_inerrs /= n_stat;
				if (isnan(avg_inerrs))
					avg_inerrs = 0.0;
				avg_outdrops /= n_stat;
				if (isnan(avg_outdrops))
					avg_outdrops = 0.0;
				printf ("<TR><TD>%s</TD><TD>%d</TD><TD>%d</TD><TD>%.1f</TD><TD>%d</TD><TD>%.1f</TD><TD>%d</TD></TR>\n",
					i_pname,
					n_down, total_down/60,
					avg_inerrs , (aggr_period * n_inerrs) / 60,
					avg_outdrops , (aggr_period * n_outdrops) / 60 );
				if( n_recs == 0 ) {
					nmatch--;
					fprintf(stderr, "%s -> %s: NO DATA AVAILABLE\n", g->g_name, ii->i_comment);
				}
				if( f != 0 )
					fclose(f);
			}
		}
	}

	printf("</TABLE>\n");

	if( nmatch == 0 && nrmatch == 0 ) {
		fprintf(stderr, "%s: no matching link/router names (or no data)\n", av0);
		exit(1);
	}
	exit(0);
}

#ifndef REGEXPINCL
/*
 * Report an error in reg expr
 */
regerr(errno)
	int errno;
{
	char *p;

	switch( errno ) {
	default:
		fprintf(stderr, "%s: error %d in regular expression\n", av0, errno);
		exit(1);

	case 11: p = "Range endpoint too large."; break;
	case 16: p = "Bad number."; break;
	case 25: p = "``\\ digit'' out of range."; break;
	case 36: p = "Illegal or missing delimiter."; break;
	case 41: p = "No remembered search string."; break;
	case 42: p = "\\( \\) imbalance."; break;
	case 43: p = "Too many \\(."; break;
	case 44: p = "More than 2 numbers given in \\{ \\}."; break;
	case 45: p = "} expected after \\."; break;
	case 46: p = "First number exceeds second in \\{ \\}."; break;
	case 49: p = "[] imbalance."; break;
	case 50: p = "Regular expression(s) too long.";
	}
	fprintf(stderr, "%s: error in regular expression: %s\n", av0, p);
	exit(1);
}
#endif /*REGEXPINCL*/

/*
 * Produce numeral suffix
 */
char *
numeral(n)
	int n;
{
	n %= 100;
	if( (n / 10) == 1 )
		return "th";
	n %= 10;
	if( n == 1 )
		return "st";
	if( n == 2 )
		return "nd";
	if( n == 3 )
		return "rd";
	return "th";
}

/*
 * Output the time period
 */
outtime(s)
	int s;
{
	struct tp {
		int udiv;
		int umax;
		char *uname;
	} *t;
	static struct tp tpp[] = {
		60,     300,    "sec",
		60,     300,    "min",
		24,     96,     "hour",
		1000000, 0,     "day",
	};

	for( t = tpp ; ; t++ ) {
		if( s < t->udiv || (s < t->umax && (s % t->udiv) != 0) ) {
			 printf("%d %s%s", s, t->uname, s > 1 ? "s" : "");
			 break;
		}
		s /= t->udiv;
	}
}

/*
 * Print the human-readable period of time.
 */
char *
saytime(t)
	u_long t;
{
	static char buf[128];
	char *bp = buf;
	int n;

#define M      60
#define H      (60*M)
#define D      (24*H)

	if( t >= D ) {
		n = t / D;
		t %= D;
		sprintf(bp, "%d day%s", n, n > 1 ? "s" : "");
		bp += strlen(buf);
		if( t > 2*H )
			t -= t % H;     /* get rid of minutes & seconds */
		else
			t -= t % M;     /* get rid of seconds */
	}
	if( t >= H ) {
		if( bp != buf )
			*bp++ = ' ';
		n = t / H;
		t %= H;
		sprintf(bp, "%d hour%s", n, n > 1 ? "s" : "");
		bp += strlen(buf);
		if( n > 3 )
			t -= t % M;     /* get rid of seconds */
	}
	if( t >= M ) {
		if( bp != buf )
			*bp++ = ' ';
		n = t / M;
		t %= M;
		sprintf(bp, "%d min%s", n, n > 1 ? "s" : "");
		bp += strlen(buf);
	}
	if( t ) {
		if( bp != buf )
			*bp++ = ' ';
		sprintf(bp, "%d sec%s", t, t > 1 ? "s" : "");
	}
	return buf;
}

/*
 * Time of the day
 */
char *
tod(t)
	u_long t;
{
	struct tm *tm = localtime(&t);
	static char buf[128];

	sprintf(buf, "%d/%d/%d %02d:%02d:%02d",
		tm->tm_mon + 1, tm->tm_mday, tm->tm_year % 100,
		tm->tm_hour, tm->tm_min, tm->tm_sec);
	return buf;
}

time_header (tt, start_tt, end_tt, yr, mo, dy, hr_start, hr_end, period)
u_long start_tt, end_tt, tt;
int yr, mo, dy, hr_start, hr_end;
enum FILE_CHANGE period;
{
	int i;
	struct tm tm;

	switch( period ) {
	case WEEKLY:
		printf("the week starting on ");
		/*FALL THRU*/

	case DAILY:
		printf("%s %d%s, %d", mon_names[mo-1], dy, numeral(dy), yr);
		if( hr_start != 0 || hr_end != 24 ) {
			i = hr_start % 12;
			printf(",  %d%s", i? i : 12, hr_start >= 12? "pm" : "am");
			i = hr_end % 12;
			printf("\\-%d%s", i? i : 12, (hr_end % 24) >= 12? "pm" : "am");
		}
		break;

	case MONTHLY:
		printf("%s, %d", mon_names[mo-1], yr);
	}
	if( end_tt > tt ) {
		tm = *localtime(&tt);
		printf(", ending");
		switch( period ) {
		case WEEKLY:
			if( mo-1 != tm.tm_mon )
				printf(" %s", mon_names[tm.tm_mon]);
			/*FALL THRU*/

		case MONTHLY:
			printf(" %d%s", tm.tm_mday, numeral(tm.tm_mday));
			if( yr - 1900 != tm.tm_year )
				printf(", %d", tm.tm_year + 1900);
			/*FALL THRU*/

		case DAILY:
			i = tm.tm_hour % 12;
			printf(" %d:%02d%s", i? i : 12, tm.tm_min,
				tm.tm_hour >= 12? "pm" : "am");
		}
	}
	printf("\n");
}
