/*
 * Copyright (C) 1993, US Sprint
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of US Sprint not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * US SPRINT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * US SPRINT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Author: Vadim Antonov
 *
 * $Log: stat.c,v $
 * Revision 1.3  1999/09/21 11:27:02  monitor
 * INACT changed
 *
 * Revision 1.2  1999/08/19 18:33:46  alex
 * The ATM/INACTIVE bug fixed
 *
 * Revision 1.1.1.1  1999/07/19 18:38:32  alex
 * First revision in cvs
 *
 * Revision 1.7  1999/05/11 15:24:01  alex
 * *** empty log message ***
 *
 * Revision 1.6  1998/03/20  19:02:40  alex
 * RELCOM
 *
 * Revision 1.5  1996/01/10  15:32:19  tim
 * *** empty log message ***
 *
 * Revision 1.4  1995/09/07  09:29:53  tim
 * average line load
 *
 * Revision 1.4  1995/09/07  09:29:53  tim
 * average line load
 *
 * Revision 1.3  1993/09/22  23:25:53  avg
 * Added logging router vitals,
 * ,
 *
 * Revision 1.3  1993/09/22  23:25:53  avg
 * Added logging router vitals,
 * ,
 *
 * Revision 1.2  1993/09/07  21:28:58  avg
 * Since the routers increment octet counters only at packet ends
 * the reported utilization may exceed the theoretical maximum
 * (but not much!).
 *
 * ,
 *
 * Revision 1.1  1993/08/10  17:47:57  avg
 * Initial revision
 *
 * Revision 1.1  1993/08/10  17:47:57  avg
 * Initial revision
 *
 */
#ifndef lint
static char rcsid[] = "$Id: stat.c,v 1.3 1999/09/21 11:27:02 monitor Exp $";
#endif

#include "manifest.h"
#include <sys/types.h>

/*
#define DELTA(x1,x2)  ((x1)<(x2)?(x2)-(x1):(x1)-(x2))
#define DELTA(x1,x2)  ((x1)<(x2)?(x1):(x1)-(x2))
*/
u_int64_t
DELTA(u_int32_t A, u_int32_t B) {
    u_int64_t a = A, b = B;
    if  (a >= b) return(a - b);
    return(a + 0x100000000ULL - b);
}

/*
 * Update bgp  statistics
 */

void
bgp_stat(g, b, nb)
	struct GATEWAY *g;
	struct BGP     *b;
	struct BGP     *nb;
{
        if(b == NULL) { bgp_opstat(g,b); return; }

	b->bgp_time = nb->bgp_time;
	b->bgp_as   = nb->bgp_as;
	if (b->bgp_state != nb->bgp_state) {
	  dprintf(("BGP state has changed for %s %s from stat %s to %s\n",
		g->g_name, b->bgp_name, bgpstats[b->bgp_state], 
		bgpstats[nb->bgp_state]));
	  b->bgp_state=nb->bgp_state;
	  bgp_opstat(g,b);
	}
}


/*
 * Update link statistics
 */
void
link_stat(g, i, l)
	struct GATEWAY *g;
	struct IF *i;
	struct LSTAT *l;
{
	double gw_td;     /* Inter-scan time delta at gateway */
	u_int64_t d;
	u_int64_t inpkt, outpkt;
	double v;
	u_int64_t maxoctets; /* Theoretical max bytes between scans */

	if ( g->g_alive != G_UP ) return; /* Can't calculate anything if gateway is unreachable */
	/* Move the old state to i_ps */
	i->i_ps = i->i_cs;
	i->i_cs = *l;
	/* If there is CHECK function, do it */
	if ( i->i_icmode && i->i_cs.l_OperStatus == IF_UP && i->i_ps.l_timestamp != 0  
	    && (i->i_icmode==IC_INACTIVE ||(i->i_icmode==IC_INACTIVE_ATM && i->i_type==ifType_aal5))
	    && ( (i->i_cs.l_InUcastPkts+i->i_cs.l_InNUcastPkts) == (i->i_ps.l_InUcastPkts + i->i_ps.l_InNUcastPkts)) 
	    && ( (i->i_icmode==IC_INACTIVE_ATM || i->i_cs.l_OutUcastPkts+i->i_cs.l_OutNUcastPkts) != (i->i_ps.l_OutUcastPkts + i->i_ps.l_OutNUcastPkts)) )
	 {
	 	i->i_cs.l_OperStatus = IF_INACTIVE; 
		if ( i->i_ps.l_OperStatus != IF_INACTIVE )
		       i->i_cs.l_LastChange = g->g_uptime;
		dprintf(("Forced to INACTIVE: %s at %s\n",i->i_name,g->g_name));
	 };
	 /* Emulate LastChange time if INACTIVE->UP */
	 if ( i->i_cs.l_OperStatus != IF_INACTIVE && i->i_ps.l_OperStatus == IF_INACTIVE )
		i->i_cs.l_LastChange = g->g_uptime;
			    

	/*  Report changes in link status */
	if( i->i_ps.l_timestamp == 0 ||
	    i->i_ps.l_OperStatus != i->i_cs.l_OperStatus ) {
		/* recode last change of list state */
		i->i_cs.l_lastchtt=i->i_cs.l_reftime-
			(g->g_uptime - i->i_cs.l_LastChange) / 100;
		link_opstat(g, i, -1, i->i_cs.l_reftime);
	}
	else i->i_cs.l_lastchtt=i->i_ps.l_lastchtt;

	/*
	 * If there is no previous scan initialize the things.
	 */
	if( i->i_ps.l_timestamp == 0 ) {
		i->i_acc.s_time = i->i_cs.l_reftime;
		return;
	}

	/* Recalculate statistical parameters */
	gw_td = ((double)(i->i_cs.l_timestamp) - (double)(i->i_ps.l_timestamp)) / (double)100.0;
	if( gw_td < 0.1 )
		gw_td = 0.1;
	if (i->i_cs.l_Speed == 0 )
	       i->i_cs.l_Speed = 9600;
	maxoctets = ((double)(i->i_cs.l_Speed) * gw_td / (double)8);

	/* InOctets avg and peak */
	d = DELTA(i->i_cs.l_InOctets, i->i_ps.l_InOctets);
	if( d > maxoctets * 2 )
		d = maxoctets;
	i->i_acc.s_inutil += d;
	if( d > maxoctets )
		d = maxoctets;
	v = 0;
	if ( maxoctets > 0 )
	       v = (100.0 * d) / maxoctets;
	if( v > i->i_acc.s_inpeakutil )
		i->i_acc.s_inpeakutil = v;

	/* In packets per second avg and peak */
	inpkt = DELTA(i->i_cs.l_InUcastPkts, i->i_ps.l_InUcastPkts) +
	    DELTA(i->i_cs.l_InNUcastPkts, i->i_ps.l_InNUcastPkts);
	i->i_acc.s_inpckts += inpkt;
	v = inpkt / gw_td;
	if( v > i->i_acc.s_inpeakpckts )
		i->i_acc.s_inpeakpckts = v;

	/* OutOctets avg and peak */
	d = DELTA(i->i_cs.l_OutOctets, i->i_ps.l_OutOctets);
	if( d > maxoctets * 2 )
		d = maxoctets;
	i->i_acc.s_oututil += d;
	if( d > maxoctets )
		d = maxoctets;
	v = 0;
	if ( maxoctets > 0 )
	       v = (100.0 * d) / maxoctets;
	if( v > i->i_acc.s_outpeakutil )
		i->i_acc.s_outpeakutil = v;

	/* Out packets per second avg and peak */
	outpkt = DELTA(i->i_cs.l_OutUcastPkts, i->i_ps.l_OutUcastPkts) +
	    DELTA(i->i_cs.l_OutNUcastPkts, i->i_ps.l_OutNUcastPkts);
	i->i_acc.s_outpckts += outpkt;
	v = outpkt / gw_td;
	if( v > i->i_acc.s_outpeakpckts )
		i->i_acc.s_outpeakpckts = v;

	/* Discards and errors */
	d = DELTA(i->i_cs.l_InErrors, i->i_ps.l_InErrors);
	if (d > inpkt)
	    d = inpkt;
	i->i_acc.s_inerrors += d;
	v = d;
	if ( inpkt > 0 )
		  v = (100.0 * d) / inpkt;
	if (v > i->i_acc.s_inpeakerrors)
		  i->i_acc.s_inpeakerrors = v;

	d = DELTA(i->i_cs.l_OutErrors, i->i_ps.l_OutErrors);
	if (d > outpkt)
	    d = outpkt;
	i->i_acc.s_outerrors += d;
	v = d;
	if ( outpkt > 0 )
	       v = (100.0 * d) / outpkt;
	if (v > i->i_acc.s_outpeakerrors)
		i->i_acc.s_outpeakerrors = v;

	d = DELTA(i->i_cs.l_InDrops, i->i_ps.l_InDrops);
	if (d > inpkt)
	    d = inpkt;
	i->i_acc.s_indrops  += d;
	v = d;
	if ( inpkt > 0 )
	    v = (100.0 * d) / inpkt;
	if (v > i->i_acc.s_inpeakdrops)
		i->i_acc.s_inpeakdrops = v;
	d = DELTA(i->i_cs.l_OutDrops, i->i_ps.l_OutDrops);
	if (d > outpkt)
	    d = outpkt;
	i->i_acc.s_outdrops += d;
	v = d;
	if ( outpkt > 0 )
	    v = (100.0 * d) / outpkt;
	if (v > i->i_acc.s_outpeakdrops)
		i->i_acc.s_outpeakdrops = v;
}

/*
 * Calculate aggregated stat parameters for the link
 */
void
link_aggr(g, i, t0)
	struct GATEWAY *g;
	struct IF *i;
	u_long t0;
{
	u_long rtd;       /* reference time difference */
	double maxoctets; /* theoretical max octets during the aggr time */
	double tip,top;   /* total input and output packets */

	/* Do nothing if we have only one scan */
	if( i->i_ps.l_timestamp == 0 )
		return;

	/*
	 * Calculate averages
	 */
	rtd = t0 - i->i_acc.s_time;
	if( rtd < 1 )
		rtd = 1;
	i->i_acc.s_time = t0;
	if ( i->i_cs.l_Speed == 0 )
	       i->i_cs.l_Speed = 9600;
	maxoctets = (i->i_cs.l_Speed / 8.0) * rtd;

	/* In utilization %% */
	if ( maxoctets > 0 )
	       i->i_acc.s_inutil /= maxoctets / 100.0;
	else
	       i->i_acc.s_inutil = 0;
	if( i->i_acc.s_inutil > i->i_acc.s_inpeakutil )
		i->i_acc.s_inutil = i->i_acc.s_inpeakutil;

	/* In packets per second */
	tip = i->i_acc.s_inpckts;
	i->i_acc.s_inpckts /= rtd;
	if( i->i_acc.s_inpckts > i->i_acc.s_inpeakpckts )
		i->i_acc.s_inpckts = i->i_acc.s_inpeakpckts;

	/* Out utilization %% */
	if ( maxoctets > 0 )
	       i->i_acc.s_oututil /= maxoctets / 100.0;
	else
	       i->i_acc.s_oututil = 0;
	if( i->i_acc.s_oututil > i->i_acc.s_outpeakutil )
		i->i_acc.s_oututil = i->i_acc.s_outpeakutil;

	/* Out packets per second */
	top = i->i_acc.s_outpckts;
	i->i_acc.s_outpckts /= rtd;
	if( i->i_acc.s_outpckts > i->i_acc.s_outpeakpckts )
		i->i_acc.s_outpckts = i->i_acc.s_outpeakpckts;

	/* Errors */
	if( i->i_acc.s_inerrors > 0.0 ) {
		i->i_acc.s_inerrors /= i->i_acc.s_inerrors + tip;
		i->i_acc.s_inerrors *= 100.0;
	}
	if( i->i_acc.s_outerrors > 0.0 ) {
		i->i_acc.s_outerrors /= i->i_acc.s_outerrors + top;
		i->i_acc.s_outerrors *= 100.0;
	}

	/* Drops */
	if( i->i_acc.s_indrops > 0.0 ) {
		i->i_acc.s_indrops /= i->i_acc.s_indrops + tip;
		i->i_acc.s_indrops *= 100.0;
	}
	if( i->i_acc.s_outdrops > 0.0 ) {
		i->i_acc.s_outdrops /= i->i_acc.s_outdrops + top;
		i->i_acc.s_outdrops *= 100.0;
	}

	/*
	 * Reset accumulators
	 */
	i->i_out = i->i_acc;
	i->i_acc.s_inutil    = i->i_acc.s_inpeakutil = 0.0L;
	i->i_acc.s_inpckts   = i->i_acc.s_inpeakpckts = 0.0L;
	i->i_acc.s_oututil   = i->i_acc.s_outpeakutil = 0.0L;
	i->i_acc.s_outpckts  = i->i_acc.s_outpeakpckts = 0.0L;
	i->i_acc.s_inerrors  = i->i_acc.s_inpeakerrors = 0.0L;
	i->i_acc.s_outerrors = i->i_acc.s_outpeakerrors = 0.0L;
	i->i_acc.s_indrops   = i->i_acc.s_inpeakdrops = 0.0L;
	i->i_acc.s_outdrops  = i->i_acc.s_outpeakdrops = 0.0L;
}

/*
 * Write logs for all gateway's interfaces and schedule update for
 * the next interface.
 */
void
outgwstat(a)
	struct SNMP_ACTION *a;
{
	struct GATEWAY *g = (struct GATEWAY *) ((char *)a -
			    (int) &((struct GATEWAY *)0)->g_updact);
	struct IF *i;
	extern u_long bof_time();

	dprintf(("***LOG: %s\n", g->g_name));

	/*
	 * If there was a file change scheduled, create a new
	 * log file for a getway with the current status record.
	 */
	if( g->g_bof != bof_time(time(0)) ) {
		gw_opstat(g, 1);
		bgp_opstat(g,NULL);
	}
		

	/* Write out statistics record */
	if( g->g_cisco )
		write_gwstat(g);

	/* Write logs for all active interfaces */
	for( i = &g->g_ifs[g->g_nifs-1] ; i >= g->g_ifs ; i-- ) {
		if( i->i_index < 0 )
			continue;
		write_lstat(g, i);

		/* Check for error rate and drops on the link */
		if( notify >= 0 && i->i_cs.l_OperStatus == IF_UP)
			error_notify(g, i);
	}

	/* Process the next gateway in two seconds */
	if( g = g->g_next ) {
		g->g_updact.a_time = time(0) + 2;
		g->g_updact.a_f = outgwstat;
		timeout(&g->g_updact);
	}
}

/*
 * Aggregate statistics and start storing log records
 */
void
aggregate(a)
	struct SNMP_ACTION *a;
{
	struct GATEWAY *g;
	struct IF *i;
	u_long t0;

	/*
	 * Process all the links
	 */
	t0 = time(0);
	for( g = gwlist ; g ; g = g->g_next ) {
		for( i = &g->g_ifs[g->g_nifs-1] ; i >= g->g_ifs ; i-- ) {
			if( i->i_index < 0 )
				continue;
			link_aggr(g, i, t0);
		}
	}

	/* Schedule the next aggregation */
	a->a_time = tsharp(a->a_time, aggr_period, 0);
	a->a_f = aggregate;
	timeout(a);

	/*
	 * Start writing to log files
	 */
	if( gwlist )
		outgwstat(&gwlist->g_updact);
}
